#
# Copyright (c) 2012-2023 Snowflake Computing Inc. All rights reserved.
#
from contextlib import suppress

import pytest

from snowflake.core.exceptions import NotFoundError
from snowflake.core.service import Service
from tests.utils import random_string


pytestmark = [pytest.mark.skip_gov]


@pytest.mark.min_sf_ver("8.37.0")
def test_create_or_alter(services, service_spec_file_on_stage, shared_compute_pool, warehouse):
    service_name = random_string(5, "test_create_or_alter_service_")

    test_service = Service(
        name=service_name,
        compute_pool=shared_compute_pool,
        spec=service_spec_file_on_stage,
        min_instances=1,
        max_instances=1,
        comment="test_service_comment",
    )

    service_ref = None

    try:
        # Test create when the Service does not exist.
        service_ref = services[service_name]
        service_ref.create_or_alter(test_service)
        service_list = services.iter(like=service_name)
        result = next(service_list)
        assert result.name == test_service.name.upper()
        assert result.compute_pool == test_service.compute_pool.upper()
        assert result.min_instances == test_service.min_instances
        assert result.max_instances == test_service.max_instances
        assert result.comment == test_service.comment

        assert result.auto_resume is True
        # Make sure that issuing an empty alter doesn't create a malformed SQL
        service_ref.create_or_alter(test_service)

        # Test introducing property which was not set before
        test_service_new_1 = Service(
            name=service_name,
            spec=service_spec_file_on_stage,
            compute_pool=shared_compute_pool,
            min_instances=1,
            max_instances=1,
            auto_resume=False,
            comment="test_service_comment",
        )
        service_ref.create_or_alter(test_service_new_1)
        service_list = services.iter(like=service_name)
        result = next(service_list)
        assert result.name == test_service_new_1.name.upper()
        assert result.compute_pool == test_service.compute_pool.upper()
        assert result.min_instances == test_service_new_1.min_instances
        assert result.max_instances == test_service_new_1.max_instances
        assert result.comment == test_service.comment
        assert result.auto_resume == test_service_new_1.auto_resume

        # Test altering the property which we set before
        test_service_new_2 = Service(
            name=service_name,
            spec=service_spec_file_on_stage,
            compute_pool=shared_compute_pool,
            min_instances=1,
            max_instances=1,
            auto_resume=False,
            comment="comment",
        )
        service_ref.create_or_alter(test_service_new_2)
        service_list = services.iter(like=service_name)
        result = next(service_list)
        assert result.name == test_service_new_2.name.upper()
        assert result.compute_pool == test_service.compute_pool.upper()
        assert result.min_instances == test_service_new_2.min_instances
        assert result.max_instances == test_service_new_2.max_instances
        assert result.comment == test_service_new_2.comment
        assert result.auto_resume == test_service_new_2.auto_resume

        # Test not providing the property and checking that it is unset now and adding a new property at the same time
        test_service_new_3 = Service(
            name=service_name,
            spec=service_spec_file_on_stage,
            compute_pool=shared_compute_pool,
            min_instances=1,
            max_instances=1,
            query_warehouse=warehouse.name,
            comment="comment",
        )
        service_ref.create_or_alter(test_service_new_3)
        service_list = services.iter(like=service_name)
        result = next(service_list)
        assert result.name == test_service_new_3.name.upper()
        assert result.compute_pool == test_service.compute_pool.upper()
        assert result.query_warehouse == test_service_new_3.query_warehouse.upper()
        assert result.min_instances == test_service_new_3.min_instances
        assert result.max_instances == test_service_new_3.max_instances
        assert result.comment == test_service_new_3.comment

        # This should be reset to the default value which is True
        assert result.auto_resume is True

    finally:
        with suppress(NotFoundError):
            service_ref.drop()



#
# Copyright (c) 2012-2023 Snowflake Computing Inc. All rights reserved.
#
import importlib

from pathlib import Path
from unittest.mock import MagicMock

import pytest

from snowflake.core._common import CreateMode, SchemaObjectCollectionParent, SchemaObjectReferenceMixin
from snowflake.core.database import DatabaseCollection
from snowflake.core.schema import SchemaResource


PROJECT_SRC = Path(__file__).parent.parent.parent / "src" / "snowflake" / "core"
DEPRECATED_METHODS = ["delete", "undelete", "create_or_update", "download_file", "upload_file"]
IGNORED_METHODS = DEPRECATED_METHODS + ["get", "put"]  # StageResource.get, StageResource.put


class XyzCollection(SchemaObjectCollectionParent["XyzReference"]):
    def __init__(self, schema: "SchemaResource") -> None:
        super().__init__(schema, XyzResource)


class XyzResource(SchemaObjectReferenceMixin):
    def __init__(self, name: str, collection: "XyzCollection") -> None:
        self.collection = collection
        self.name = name


class ArgsCollection(SchemaObjectCollectionParent["ArgsResource"]):
    def __init__(self, schema: "SchemaResource") -> None:
        super().__init__(schema, ArgsResource)


class ArgsResource(SchemaObjectReferenceMixin):
    def __init__(self, name_with_args: str, collection: "ArgsCollection") -> None:
        self.collection = collection
        self.name_with_args = name_with_args


def test_collection_and_references():
    mock_session = MagicMock()
    db_collection = DatabaseCollection(mock_session)
    my_db_ref = db_collection["my_db"]
    assert my_db_ref.name == "my_db"
    assert my_db_ref.schemas.database.collection is db_collection

    schema_collection = my_db_ref.schemas
    my_schema_ref = schema_collection["my_schema"]
    assert my_schema_ref.name == "my_schema"
    assert my_schema_ref.database.name == "my_db"
    assert my_schema_ref.collection is schema_collection
    assert my_schema_ref.collection.database is my_db_ref is my_schema_ref.database
    assert my_schema_ref.qualified_name == "my_db.my_schema"

    xyz_collection = XyzCollection(my_schema_ref)
    my_xyz_ref = xyz_collection["my_xyz"]

    assert my_xyz_ref.name == "my_xyz"
    assert my_xyz_ref.schema.name == "my_schema"
    assert my_xyz_ref.database.name == "my_db"
    assert my_xyz_ref.schema is my_schema_ref
    assert my_xyz_ref.database is my_db_ref
    assert my_xyz_ref.collection is xyz_collection
    assert my_xyz_ref.fully_qualified_name == "my_db.my_schema.my_xyz"

    for key in xyz_collection:
        assert key == "my_xyz"

    for key in xyz_collection.keys():
        assert key == "my_xyz"

    for item in xyz_collection.items():
        assert item[0] == "my_xyz"
        assert item[1] is my_xyz_ref

    for value in xyz_collection.values():
        assert value is my_xyz_ref


def test_repr():
    db_collection = DatabaseCollection(MagicMock())
    my_db_ref = db_collection["my_db"]
    assert repr(db_collection) == "<DatabaseCollection>"
    assert repr(my_db_ref) == "<DatabaseResource: 'my_db'>"

    schema_collection = my_db_ref.schemas
    my_schema_ref = schema_collection['"my_schema"']
    assert repr(schema_collection) == "<SchemaCollection: 'my_db'>"
    assert repr(my_schema_ref) == """<SchemaResource: 'my_db."my_schema"'>"""

    xyz_collection = XyzCollection(my_schema_ref)
    my_xyz_ref = xyz_collection["my_xyz"]
    assert repr(xyz_collection) == """<XyzCollection: 'my_db."my_schema"'>"""
    assert repr(my_xyz_ref) == """<XyzResource: 'my_db."my_schema".my_xyz'>"""

    args_collection = ArgsCollection(my_schema_ref)
    my_args_ref = args_collection["my_args(float)"]
    assert repr(args_collection) == """<ArgsCollection: 'my_db."my_schema"'>"""
    assert repr(my_args_ref) == """<ArgsResource: 'my_db."my_schema".my_args(float)'>"""


def test_createmode():
    assert CreateMode["orREPlace"] is CreateMode.or_replace
    assert CreateMode["orREPlace"] == CreateMode.or_replace.value
    assert CreateMode["ifNOTExists"] is CreateMode.if_not_exists
    assert CreateMode["ErroRifexists"] is CreateMode.error_if_exists


def _generate_all_resources():
    for module in Path(PROJECT_SRC).iterdir():
        if module.name.startswith("_") or module.is_file():
            continue
        if module.name in [
            "cortex",  # Needs getting into underlying package
            "cortex_analyst",
            "session",  # Session is not resource
        ]:
            continue

        import_path = ".".join(module.parts[-3:])
        resource_module = importlib.import_module(import_path)
        yield resource_module, module.name


def _generate_all_classes(suffix=""):
    for resource_module, module_name in _generate_all_resources():
        class_name = "".join(s.capitalize() for s in module_name.split("_")) + suffix
        if hasattr(resource_module, class_name):
            a_class = getattr(resource_module, class_name)
            yield a_class


@pytest.mark.parametrize("model_class", _generate_all_classes())
def test_all_resources_have_to_dict_method(model_class):
    assert hasattr(model_class, "to_dict"), f"{model_class.__name__} missing to_dict method"


@pytest.mark.parametrize("collection_class", _generate_all_classes(suffix="Collection"))
@pytest.mark.parametrize("method", ("create", "iter"))
def test_collection_defines_async_methods(collection_class, method):
    method_name = f"{method}_async"
    assert hasattr(collection_class, method_name), f"{collection_class.__name__} missing {method_name} method"


@pytest.mark.parametrize("resource_class", _generate_all_classes(suffix="Resource"))
def test_resource_defines_async_methods(resource_class):
    method_names = (
        f"{m}_async"
        for m in dir(resource_class)
        if callable(getattr(resource_class, m))
        and not m.startswith("_")
        and not m.endswith("async")
        and m not in IGNORED_METHODS
    )
    for method_name in method_names:
        assert hasattr(resource_class, method_name), f"{resource_class.__name__} missing {method_name} method"



"""Integration tests for preview feature filtering."""

import pytest
from datetime import datetime


class TestGAEndpoints:
    def test_ga_endpoints_exist(self, default_api_class):
        assert hasattr(default_api_class, "get_resource")
        assert hasattr(default_api_class, "create_resource")
        assert hasattr(default_api_class, "get_resource_advanced")

        assert callable(getattr(default_api_class, "get_resource"))
        assert callable(getattr(default_api_class, "create_resource"))
        assert callable(getattr(default_api_class, "get_resource_advanced"))


class TestPreviewEndpoints:
    def test_preview_endpoints_not_exist(self, default_api_class):
        assert not hasattr(default_api_class, "get_preview_feature")
        assert not hasattr(default_api_class, "create_experimental_feature")
        assert not hasattr(default_api_class, "get_experimental_shared")
        assert not hasattr(default_api_class, "create_composed_preview")


class TestGAModels:
    def test_ga_models_exist(self, models_module):
        assert hasattr(models_module, "Resource")
        assert hasattr(models_module, "CreateResourceRequest")
        assert hasattr(models_module, "SharedModel")


class TestPreviewModels:
    def test_preview_models_not_exist(self, models_module):
        assert not hasattr(models_module, "PreviewFeatureResponse")
        assert not hasattr(models_module, "ExperimentalRequest")
        assert not hasattr(models_module, "ExperimentalResponse")

    def test_nested_preview_models_not_exist(self, models_module):
        assert not hasattr(models_module, "NestedPreviewModel")
        assert not hasattr(models_module, "DeeplyNestedPreviewModel")

    def test_allof_preview_models_not_exist(self, models_module):
        assert not hasattr(models_module, "ComposedPreviewRequest")
        assert not hasattr(models_module, "ComposedPreviewResponse")
        assert not hasattr(models_module, "PreviewBaseModel")
        assert not hasattr(models_module, "AnotherPreviewBase")


class TestModelFunctionality:
    def test_create_resource_request(self, models_module):
        CreateResourceRequest = models_module.CreateResourceRequest

        request = CreateResourceRequest(name="my_resource")
        assert request.name == "my_resource"
        assert request.description is None

        request_full = CreateResourceRequest(name="my_resource", description="Test resource")
        assert request_full.name == "my_resource"
        assert request_full.description == "Test resource"

    def test_resource_model(self, models_module):
        Resource = models_module.Resource

        resource = Resource(name="test_db", status="active", created_at="2024-01-01T00:00:00Z")

        assert resource.name == "test_db"
        assert resource.status == "active"
        assert isinstance(resource.created_at, datetime)
        assert resource.created_at.year == 2024
        assert resource.created_at.month == 1
        assert resource.created_at.day == 1

    def test_shared_model(self, models_module):
        SharedModel = models_module.SharedModel

        model = SharedModel(shared_field="test_value")
        assert model.shared_field == "test_value"



import pytest

from snowflake.core.event_table import EventTable
from snowflake.core.exceptions import NotFoundError
from tests.utils import random_string


@pytest.mark.min_sf_ver("8.35.0")
def test_drop(event_tables):
    event_table_name = random_string(10, "test_drop_event_table_")

    event_table_handle = event_tables.create(EventTable(name=event_table_name))

    event_table_handle.fetch()

    event_table_handle.drop()

    with pytest.raises(NotFoundError):
        event_table_handle.fetch()

    with pytest.raises(NotFoundError):
        event_table_handle.drop()

    event_table_handle.drop(if_exists=True)



#
# Copyright (c) 2012-2023 Snowflake Computing Inc. All rights reserved.
#
import pytest

from pydantic_core._pydantic_core import ValidationError

from tests.utils import unquote


pytestmark = [pytest.mark.usefixtures("backup_database_schema")]


def test_iter(databases, temp_db, temp_db_case_sensitive):
    database_names = [db.name for db in databases.iter()]
    assert temp_db.name.upper() in database_names
    assert unquote(temp_db_case_sensitive.name) in database_names


def test_iter_like(databases, temp_db, temp_db_case_sensitive):
    database_names = [db.name for db in databases.iter(like="test_database%")]
    assert temp_db.name.upper() in database_names
    assert unquote(temp_db_case_sensitive.name) in database_names


def test_iter_starts_with(databases, temp_db, temp_db_case_sensitive):
    database_names = [db.name for db in databases.iter(starts_with="Test_database")]
    assert temp_db.name.upper() not in database_names
    assert unquote(temp_db_case_sensitive.name) not in database_names

    database_names = [db.name for db in databases.iter(starts_with="TEST_DATABASE")]
    assert temp_db.name.upper() in database_names
    assert unquote(temp_db_case_sensitive.name) not in database_names


# The limit keyword is required for the from keyword to function, limit=100 was chosen arbitrarily
# as it does not affect the test
def test_iter_from_name(databases, temp_db, temp_db_case_sensitive):
    database_names = [db.name for db in databases.iter(limit=100, from_name="test_database")]
    assert temp_db.name.upper() not in database_names
    assert unquote(temp_db_case_sensitive.name) in database_names


def test_iter_limit(databases):
    data = list(databases.iter())
    initial_length = min(len(data), 10000)

    data = list(databases.iter(limit=initial_length))
    assert len(data) <= initial_length

    data = list(databases.iter(limit=initial_length - 1))
    assert len(data) <= initial_length - 1

    data = list(databases.iter(limit=10000))
    assert len(data) <= 10000

    with pytest.raises(ValidationError):
        list(databases.iter(limit=10001))



import pytest

from snowflake.core.exceptions import NotFoundError
from snowflake.core.grant._grant import Grant
from snowflake.core.grant._grantee import Grantees
from snowflake.core.grant._privileges import Privileges
from snowflake.core.grant._securables import Securables
from snowflake.core.role import Role
from tests.utils import random_string


@pytest.mark.use_accountadmin
def test_apply_grant(
    grants, database, schema, test_database_role_name, test_share_name, test_user_name, test_table_name, test_role_name
):
    # grants to current account
    grants.grant(
        Grant(
            grantee=Grantees.role(name=test_role_name),
            securable=Securables.current_account,
            privileges=[
                Privileges.create_database,
                Privileges.create_compute_pool,
                Privileges.create_warehouse,
                Privileges.create_account,
            ],
        )
    )

    # grants to database
    grants.grant(
        Grant(
            grantee=Grantees.role(name=test_role_name),
            securable=Securables.database(database.name),
            privileges=[Privileges.modify, Privileges.monitor],
        )
    )

    # grants to schema
    grants.grant(
        Grant(
            grantee=Grantees.role(name=test_role_name),
            securable=Securables.schema(schema.name),
            privileges=[Privileges.create_task],
        )
    )

    # grants to all schemas in database
    grants.grant(
        Grant(
            grantee=Grantees.role(name=test_role_name),
            securable=Securables.all_schemas(Securables.database(database.name)),
            privileges=[Privileges.usage],
        )
    )

    # grants to future schemas in database
    grants.grant(
        Grant(
            grantee=Grantees.role(name=test_role_name),
            securable=Securables.future_schemas(Securables.database(database.name)),
            privileges=[Privileges.create_table],
        )
    )

    # grants to table
    grants.grant(
        Grant(
            grantee=Grantees.role(name=test_role_name),
            securable=Securables.table(test_table_name),
            privileges=[Privileges.select, Privileges.insert, Privileges.delete],
        )
    )

    # grants to all tables in schema
    grants.grant(
        Grant(
            grantee=Grantees.role(name=test_role_name),
            securable=Securables.all_tables(Securables.schema(schema.name)),
            privileges=[Privileges.insert],
        )
    )

    # grants to future tables in schema
    grants.grant(
        Grant(
            grantee=Grantees.role(name=test_role_name),
            securable=Securables.future_tables(Securables.schema(schema.name)),
            privileges=[Privileges.delete],
        )
    )

    # grants create schema to database
    grants.grant(
        Grant(
            grantee=Grantees.role(name=test_role_name),
            securable=Securables.database(database.name),
            privileges=[Privileges.create_schema],
        )
    )

    # grants create task to schema
    grants.grant(
        Grant(
            grantee=Grantees.database_role(name=test_database_role_name),
            securable=Securables.schema(schema.name),
            privileges=[Privileges.create_task],
        )
    )

    # grants create task to schema
    grants.grant(
        Grant(
            grantee=Grantees.database_role(name=test_database_role_name),
            securable=Securables.schema(schema.name),
            privileges=[Privileges.create_task],
        )
    )

    grants.grant(
        Grant(
            grantee=Grantees.database_role(name=test_database_role_name),
            securable=Securables.all_schemas(Securables.database(database.name)),
            privileges=[Privileges.usage],
        )
    )

    grants.grant(
        Grant(
            grantee=Grantees.database_role(name=test_database_role_name),
            securable=Securables.future_schemas(Securables.database(database.name)),
            privileges=[Privileges.create_table],
        )
    )

    grants.grant(
        Grant(
            grantee=Grantees.database_role(name=test_database_role_name),
            securable=Securables.table(test_table_name),
            privileges=[Privileges.select],
            grant_option=True,
        )
    )

    grants.grant(
        Grant(
            grantee=Grantees.database_role(name=test_database_role_name),
            securable=Securables.all_tables(Securables.schema(schema.name)),
            privileges=[Privileges.insert],
        )
    )

    grants.grant(
        Grant(
            grantee=Grantees.database_role(name=test_database_role_name),
            securable=Securables.future_tables(Securables.schema(schema.name)),
            privileges=[Privileges.delete],
        )
    )

    grants.grant(
        Grant(
            grantee=Grantees.share(name=test_share_name),
            securable=Securables.database(database.name),
            privileges=[Privileges.usage],
        )
    )

    grants.grant(
        Grant(
            grantee=Grantees.share(name=test_share_name),
            securable=Securables.schema(schema.name),
            privileges=[Privileges.usage],
        )
    )

    grants.grant(
        Grant(
            grantee=Grantees.share(name=test_share_name),
            securable=Securables.table(test_table_name),
            privileges=[Privileges.select],
        )
    )

    grants.grant(
        Grant(
            grantee=Grantees.share(name=test_share_name),
            securable=Securables.all_iceberg_tables(Securables.schema(schema.name)),
            privileges=[Privileges.select],
        )
    )

    grants.grant(
        Grant(
            grantee=Grantees.share(name=test_share_name),
            securable=Securables.database(database.name),
            privileges=[Privileges.reference_usage],
        )
    )

    grants.grant(Grant(grantee=Grantees.user(name=test_user_name), securable=Securables.role(test_role_name)))

    grants.grant(
        Grant(
            grantee=Grantees.role(name=test_role_name), securable=Securables.database_role(name=test_database_role_name)
        )
    )


@pytest.mark.use_accountadmin
def test_apply_grant_with_grant_opt(grants, test_role_name):
    grants.grant(
        Grant(
            grantee=Grantees.role(name=test_role_name),
            securable=Securables.current_account,
            privileges=[Privileges.create_database],
            grant_option=False,
        )
    )


@pytest.mark.use_accountadmin
def test_grant_role_to_another_role(roles, grants, session):
    role_one = random_string(4, "test_grant_role_")
    role_two = random_string(4, "test_grant_role_")

    try:
        for role in [role_one, role_two]:
            roles.create(Role(name=role, comment="test_comment"))

        grants.grant(Grant(grantee=Grantees.role(role_one), securable=Securables.role(role_two)))

    finally:
        session.sql(f"DROP ROLE IF EXISTS {role_one}")
        session.sql(f"DROP ROLE IF EXISTS {role_two}")


def test_grants_for_invalid_role_names(grants):
    with pytest.raises(NotFoundError):
        grants.grant(
            Grant(
                grantee=Grantees.role(name='"some-random-role"'),
                securable=Securables.current_account,
                privileges=[Privileges.create_database],
                grant_option=False,
            )
        )


def test_grants_for_invalid_db_securable(grants):
    with pytest.raises(NotFoundError):
        grants.grant(
            Grant(
                grantee=Grantees.role(name="public"),
                securable=Securables.database("invaliddb123"),
                privileges=[Privileges.create_database],
                grant_option=False,
            )
        )



import typing

import pytest

from snowflake.core import CreateMode
from snowflake.core.exceptions import NotFoundError
from snowflake.core.user import User, UserCollection
from tests.utils import random_string


if typing.TYPE_CHECKING:
    from snowflake.snowpark import Session


@pytest.mark.snowpark
@pytest.mark.use_accountadmin
def test_delete_user(users: UserCollection, session: "Session"):
    user_name = random_string(5, "test_create_user_1")
    try:
        test_user = User(name=user_name, comment="test_comment")

        user_ref = users.create(test_user, mode=CreateMode.error_if_exists)
        assert user_ref.name == user_name
        user_ref.drop()

        with pytest.raises(NotFoundError):
            users[user_name].drop(if_exists=False)
            users[user_name].fetch()

        user_ref.drop(if_exists=True)
    finally:
        session.sql(f"DROP USER IF EXISTS {user_name}").collect()



"""
Snowflake Stage API.

The Snowflake Stage API is a REST API that you can use to access, update, and perform certain actions on stage resources in a Snowflake database.  # noqa: E501

The version of the OpenAPI document: 0.0.1
Contact: support@snowflake.com
Generated by: https://openapi-generator.tech

Do not edit this file manually.
"""

from __future__ import annotations

import json
import pprint
import re  # noqa: F401

from typing import Any, Optional

from pydantic import ConfigDict, SecretStr, StrictStr

from snowflake.core.stage._generated.models.credentials import Credentials


class AwsCredentials(Credentials):
    """A model object representing the AwsCredentials resource.

    Constructs an object of type AwsCredentials with the provided properties.

    Parameters
    __________
    aws_key_id : str, optional
        Scoped credentials generated by AWS Security Token Service (STS).
    aws_secret_key : str, optional
        Scoped credentials generated by AWS Security Token Service (STS).
    aws_token : str, optional
        Scoped credentials generated by AWS Security Token Service (STS).
    aws_role : str, optional
        Omit the security credentials and access keys and, instead, identify the role using AWS_ROLE and specify the AWS role ARN (Amazon Resource Name).
    """

    aws_key_id: Optional[SecretStr] = None

    aws_secret_key: Optional[SecretStr] = None

    aws_token: Optional[SecretStr] = None

    aws_role: Optional[StrictStr] = None

    __properties = ["credential_type"]

    model_config = ConfigDict(
        validate_by_name=True,
        validate_assignment=True,
    )

    def to_str(self) -> str:
        """Returns the string representation of the model using alias."""
        return pprint.pformat(self.dict(by_alias=True))

    def to_json(self) -> str:
        """Returns the JSON representation of the model using alias."""
        return json.dumps(self.to_dict())

    @classmethod
    def from_json(cls, json_str: str) -> AwsCredentials:
        """Create an instance of AwsCredentials from a JSON string."""
        return cls.from_dict(json.loads(json_str))

    def to_dict(
        self,
        hide_readonly_properties: bool = False,
    ) -> dict[str, Any]:
        """Returns the dictionary representation of the model using alias."""
        exclude_properties = set()

        if hide_readonly_properties:
            exclude_properties.update({})

        _dict = self.model_dump(serialize_as_any=True, by_alias=True, exclude=exclude_properties, exclude_none=True)

        _dict["credential_type"] = Credentials.get_child_model_discriminator_value("AwsCredentials")

        return _dict

    def to_dict_without_readonly_properties(self) -> dict[str, Any]:
        """Return the dictionary representation of the model without readonly properties."""
        return self.to_dict(hide_readonly_properties=True)

    @classmethod
    def from_dict(cls, obj: dict) -> AwsCredentials:
        """Create an instance of AwsCredentials from a dict."""
        if obj is None:
            return None

        if type(obj) is not dict:
            return AwsCredentials.model_validate(obj)

        _obj = AwsCredentials.model_validate(
            {
                "aws_key_id": obj.get("aws_key_id"),
                "aws_secret_key": obj.get("aws_secret_key"),
                "aws_token": obj.get("aws_token"),
                "aws_role": obj.get("aws_role"),
            }
        )

        return _obj


class AwsCredentialsModel(Credentials):
    def __init__(
        self,  # optional properties
        aws_key_id: Optional[str] = None,
        aws_secret_key: Optional[str] = None,
        aws_token: Optional[str] = None,
        aws_role: Optional[str] = None,
    ):
        """A model object representing the AwsCredentials resource.

        Constructs an object of type AwsCredentials with the provided properties.

        Parameters
        __________
        aws_key_id : str, optional
            Scoped credentials generated by AWS Security Token Service (STS).
        aws_secret_key : str, optional
            Scoped credentials generated by AWS Security Token Service (STS).
        aws_token : str, optional
            Scoped credentials generated by AWS Security Token Service (STS).
        aws_role : str, optional
            Omit the security credentials and access keys and, instead, identify the role using AWS_ROLE and specify the AWS role ARN (Amazon Resource Name).
        """
        super().__init__()
        self.aws_key_id = aws_key_id
        self.aws_secret_key = aws_secret_key
        self.aws_token = aws_token
        self.aws_role = aws_role

    __properties = ["credential_type"]

    def __repr__(self) -> str:
        return repr(self._to_model())

    def _to_model(self):
        return AwsCredentials(
            aws_key_id=self.aws_key_id,
            aws_secret_key=self.aws_secret_key,
            aws_token=self.aws_token,
            aws_role=self.aws_role,
        )

    @classmethod
    def _from_model(cls, model) -> AwsCredentialsModel:
        return AwsCredentialsModel(
            aws_key_id=model.aws_key_id,
            aws_secret_key=model.aws_secret_key,
            aws_token=model.aws_token,
            aws_role=model.aws_role,
        )

    def to_dict(self):
        """Create a dictionary of the properties from a AwsCredentials.

        This method constructs a dictionary with the key-value entries corresponding to the properties of the AwsCredentials object.

        Returns
        _______
        dict
            A dictionary object created using the input model.
        """
        return self._to_model().to_dict()

    @classmethod
    def from_dict(cls, obj: dict) -> AwsCredentialsModel:
        """Create an instance of AwsCredentials from a dict.

        This method constructs a AwsCredentials object from a dictionary with the key-value pairs of its properties.

        Parameters
        ----------
        obj : dict
            A dictionary whose keys and values correspond to the properties of the resource object.

        Returns
        _______
        AwsCredentials
            A AwsCredentials object created using the input dictionary; this will fail if the required properties are missing.
        """
        return cls._from_model(AwsCredentials.from_dict(obj))


AwsCredentials._model_class = AwsCredentialsModel



from snowflake.core.sequence import Sequence


test_sequence_template = Sequence(
    name="to_be_set",
    start=2,
    increment=2,
    ordered=True,
    comment="Test sequence",
)

test_sequence_minimal_template = Sequence(
    name="to_be_set",
)



#
# Copyright (c) 2012-2023 Snowflake Computing Inc. All rights reserved.
#
import copy
import json
import time

import pytest

from snowflake.core import Clone, CreateMode, PointOfTimeOffset
from snowflake.core.database import Database, DatabaseCollection
from snowflake.core.exceptions import APIError, ConflictError, UnauthorizedError
from tests.utils import random_string, unquote


pytestmark = pytest.mark.usefixtures("backup_database_schema")


def test_create_database(databases: DatabaseCollection):
    new_db_def = Database(name=random_string(3, "test_database_$12create_"), kind="TRANSIENT")
    new_db_def.comment = "database first"
    database = databases.create(new_db_def)
    try:
        created_database = database.fetch()
        assert created_database.name == new_db_def.name.upper()
        assert created_database.kind == "TRANSIENT"
        assert created_database.comment == new_db_def.comment

        with pytest.raises(ConflictError):
            databases.create(new_db_def, mode=CreateMode.error_if_exists)

        new_db_def_1 = copy.deepcopy(new_db_def)
        new_db_def_1.comment = "databse second"
        new_db_def_1.kind = None
        database = databases.create(new_db_def_1, mode=CreateMode.if_not_exists)

        created_database = database.fetch()
        assert created_database.name == new_db_def.name.upper()
        assert created_database.kind == "TRANSIENT"
        assert created_database.comment == new_db_def.comment
    finally:
        database.drop()

    try:
        database = databases.create(new_db_def_1, mode=CreateMode.or_replace)

        created_database = database.fetch()
        assert created_database.name == new_db_def_1.name.upper()
        assert created_database.kind == "PERMANENT"
        assert created_database.comment == new_db_def_1.comment
    finally:
        database.drop()

    try:
        database_name = random_string(10, "test_database_INT_test_")
        database_name_case_sensitive = '"' + database_name + '"'
        new_db_def = Database(name=database_name_case_sensitive)
        database = databases.create(new_db_def)
        created_database = database.fetch()
        assert created_database.name == unquote(new_db_def.name)
    finally:
        database.drop()

    try:
        database_name = random_string(10, 'test_database_""INT""_test_#_')
        database_name_case_sensitive = '"' + database_name + '"'
        new_db_def = Database(name=database_name_case_sensitive)
        database = databases.create(new_db_def)
        created_database = database.fetch()
        assert created_database.name == unquote(new_db_def.name)
    finally:
        database.drop()


@pytest.mark.min_sf_ver("9.8.0")
def test_create_clone(databases: DatabaseCollection):
    database_name = random_string(3, "test_database_")
    database_def = Database(name=database_name, kind="TRANSIENT")

    new_database_name = random_string(3, "test_database_clone_")
    new_database_def = Database(name=new_database_name)

    # error because Database does not exist
    with pytest.raises(APIError):
        databases.create(
            new_database_def,
            clone=Clone(source=database_name, point_of_time=PointOfTimeOffset(reference="before", when="-5")),
            mode=CreateMode.or_replace,
        )

    databases.create(database_def)
    with pytest.raises(APIError, match="transient object cannot be cloned to a permanent object"):
        databases.create(
            new_database_def,
            clone=Clone(source=database_name, point_of_time=PointOfTimeOffset(reference="before", when="-1")),
            mode=CreateMode.or_replace,
        )

    # can clone transient to transient
    new_database_def.kind = "TRANSIENT"
    # sleep is needed due to using point_of_time in clone
    time.sleep(2)
    databases.create(
        new_database_def,
        clone=Clone(source=database_name, point_of_time=PointOfTimeOffset(reference="at", when="-1")),
        mode=CreateMode.or_replace,
    )

    # replaced transient to permanent database
    new_database_def.kind = database_def.kind = "PERMANENT"
    databases.create(database_def, mode=CreateMode.or_replace)
    db = databases.create(new_database_def, clone=Clone(source=database_name), mode=CreateMode.or_replace)
    try:
        db.fetch()
    finally:
        db.drop()

    # clone database setting optional attributes on the clone
    new_database_def.serverless_task_min_statement_size = "SMALL"
    new_database_def.serverless_task_max_statement_size = "MEDIUM"
    new_database_def.user_task_managed_initial_warehouse_size = "SMALL"
    new_database_def.suspend_task_after_num_failures = 2
    new_database_def.user_task_timeout_ms = 20000
    databases.create(database_def, mode=CreateMode.or_replace)
    db = databases.create(new_database_def, clone=Clone(source=database_name), mode=CreateMode.or_replace)
    try:
        fetched_db = db.fetch()
        assert fetched_db.serverless_task_min_statement_size == "SMALL"
        assert fetched_db.serverless_task_max_statement_size == "MEDIUM"
        assert fetched_db.user_task_managed_initial_warehouse_size == "SMALL"
        assert fetched_db.suspend_task_after_num_failures == 2
        assert fetched_db.user_task_timeout_ms == 20000
    finally:
        db.drop()


@pytest.mark.usefixtures("shared_database_available")
def test_create_from_share(databases: DatabaseCollection):
    new_db_name = random_string(3, "test_db_from_share_")

    try:
        db = databases.create(Database(name=new_db_name), from_share='SFSALESSHARED.SFC_SAMPLES_PROD3."SAMPLE_DATA"')

        try:
            assert db.fetch().is_current
        finally:
            db.drop()

    except UnauthorizedError as err:
        # We can't import this more than once if another test has already imported it.
        # So, catch that case and pass if that is detected.
        assert "importing more than once is not supported" in json.loads(err.body)["message"]
        pytest.skip("Test was not run because of database was already shared.")



"""
Snowflake Stage API.

The Snowflake Stage API is a REST API that you can use to access, update, and perform certain actions on stage resources in a Snowflake database.  # noqa: E501

The version of the OpenAPI document: 0.0.1
Contact: support@snowflake.com
Generated by: https://openapi-generator.tech

Do not edit this file manually.
"""

from __future__ import annotations

import json
import pprint
import re  # noqa: F401

from typing import Any, Optional

from pydantic import ConfigDict, SecretStr

from snowflake.core.stage._generated.models.credentials import Credentials


class AzureCredentials(Credentials):
    """A model object representing the AzureCredentials resource.

    Constructs an object of type AzureCredentials with the provided properties.

    Parameters
    __________
    azure_sas_token : str, optional
        Specifies the SAS (shared access signature) token for connecting to Azure.
    """

    azure_sas_token: Optional[SecretStr] = None

    __properties = ["credential_type"]

    model_config = ConfigDict(
        validate_by_name=True,
        validate_assignment=True,
    )

    def to_str(self) -> str:
        """Returns the string representation of the model using alias."""
        return pprint.pformat(self.dict(by_alias=True))

    def to_json(self) -> str:
        """Returns the JSON representation of the model using alias."""
        return json.dumps(self.to_dict())

    @classmethod
    def from_json(cls, json_str: str) -> AzureCredentials:
        """Create an instance of AzureCredentials from a JSON string."""
        return cls.from_dict(json.loads(json_str))

    def to_dict(
        self,
        hide_readonly_properties: bool = False,
    ) -> dict[str, Any]:
        """Returns the dictionary representation of the model using alias."""
        exclude_properties = set()

        if hide_readonly_properties:
            exclude_properties.update({})

        _dict = self.model_dump(serialize_as_any=True, by_alias=True, exclude=exclude_properties, exclude_none=True)

        _dict["credential_type"] = Credentials.get_child_model_discriminator_value("AzureCredentials")

        return _dict

    def to_dict_without_readonly_properties(self) -> dict[str, Any]:
        """Return the dictionary representation of the model without readonly properties."""
        return self.to_dict(hide_readonly_properties=True)

    @classmethod
    def from_dict(cls, obj: dict) -> AzureCredentials:
        """Create an instance of AzureCredentials from a dict."""
        if obj is None:
            return None

        if type(obj) is not dict:
            return AzureCredentials.model_validate(obj)

        _obj = AzureCredentials.model_validate(
            {
                "azure_sas_token": obj.get("azure_sas_token"),
            }
        )

        return _obj


class AzureCredentialsModel(Credentials):
    def __init__(
        self,  # optional properties
        azure_sas_token: Optional[str] = None,
    ):
        """A model object representing the AzureCredentials resource.

        Constructs an object of type AzureCredentials with the provided properties.

        Parameters
        __________
        azure_sas_token : str, optional
            Specifies the SAS (shared access signature) token for connecting to Azure.
        """
        super().__init__()
        self.azure_sas_token = azure_sas_token

    __properties = ["credential_type"]

    def __repr__(self) -> str:
        return repr(self._to_model())

    def _to_model(self):
        return AzureCredentials(
            azure_sas_token=self.azure_sas_token,
        )

    @classmethod
    def _from_model(cls, model) -> AzureCredentialsModel:
        return AzureCredentialsModel(
            azure_sas_token=model.azure_sas_token,
        )

    def to_dict(self):
        """Create a dictionary of the properties from a AzureCredentials.

        This method constructs a dictionary with the key-value entries corresponding to the properties of the AzureCredentials object.

        Returns
        _______
        dict
            A dictionary object created using the input model.
        """
        return self._to_model().to_dict()

    @classmethod
    def from_dict(cls, obj: dict) -> AzureCredentialsModel:
        """Create an instance of AzureCredentials from a dict.

        This method constructs a AzureCredentials object from a dictionary with the key-value pairs of its properties.

        Parameters
        ----------
        obj : dict
            A dictionary whose keys and values correspond to the properties of the resource object.

        Returns
        _______
        AzureCredentials
            A AzureCredentials object created using the input dictionary; this will fail if the required properties are missing.
        """
        return cls._from_model(AzureCredentials.from_dict(obj))


AzureCredentials._model_class = AzureCredentialsModel



#
# Copyright (c) 2012-2023 Snowflake Computing Inc. All rights reserved.
#
import pytest

from snowflake.core.compute_pool import ComputePool
from tests.utils import random_string


pytestmark = [pytest.mark.skip_gov]


def test_fetch(compute_pools, temp_cp, instance_family):
    cp_ref = compute_pools[temp_cp.name]

    # testing with correct instance name
    # check default values
    cp = cp_ref.fetch()
    assert (
        cp.name == temp_cp.name.upper()  # for upper/lower case names
    )
    assert cp.min_nodes == 1
    assert cp.max_nodes == 1
    assert cp.created_on
    assert cp.comment == "created by temp_cp"
    assert not cp.auto_resume
    assert cp.auto_suspend_secs == 3600
    assert cp.num_services is not None
    assert cp.num_jobs is not None
    assert cp.active_nodes is not None
    assert cp.state is not None
    # TODO(SNOW-1707268): Uncomment the line below once the issue is fixed
    # assert cp.idle_nodes is not None
    assert cp.target_nodes is not None
    assert cp.resumed_on is None
    assert cp.updated_on is not None
    assert cp.owner is not None
    assert cp.is_exclusive is not None

    # check the set properties
    cp_name = random_string(5, "test_cp_")
    test_cp = ComputePool(
        name=cp_name,
        instance_family=instance_family,
        min_nodes=1,
        max_nodes=5,
        comment="created by test_cp",
        auto_resume=True,
        auto_suspend_secs=500,
    )
    try:
        cp_ref = compute_pools.create(test_cp, initially_suspended=True)
        cp = cp_ref.fetch()
        assert cp.name == test_cp.name.upper()
        assert cp.min_nodes == 1
        assert cp.max_nodes == 5
        assert cp.created_on
        assert cp.comment == "created by test_cp"
        assert cp.auto_resume
        assert cp.state in ("SUSPENDED", "STOPPING")
        assert cp.auto_suspend_secs == 500
    finally:
        cp_ref.drop()



from decimal import Decimal

import pytest

from snowflake.core._utils import fix_hostname, map_result, replace_function_name_in_name_with_args
from snowflake.core.procedure import Procedure, ReturnDataType, SQLFunction


def test_replace_function_name_in_name_with_args():
    assert (
        replace_function_name_in_name_with_args("function_name(arg1,arg2)", "new_function_name")
        == "new_function_name(arg1,arg2)"
    )

    assert replace_function_name_in_name_with_args("function_name()", "new_function_name") == "new_function_name()"

    assert (
        replace_function_name_in_name_with_args("""\"()fun()\"()""", """new_function_name""") == "new_function_name()"
    )

    assert (
        replace_function_name_in_name_with_args("""\"()fun()\"(ar)""", """new_function_name""")
        == "new_function_name(ar)"
    )

    assert (
        replace_function_name_in_name_with_args("""\"()fun()\"(ar12)""", """new_function_name""")
        == "new_function_name(ar12)"
    )

    assert (
        replace_function_name_in_name_with_args("""\"()fun()\"(ar12,ar13)""", """\"()()()\"""") == '"()()()"(ar12,ar13)'
    )

    assert replace_function_name_in_name_with_args("""abc(ar12,ar13)""", """\"()()()\"""") == '"()()()"(ar12,ar13)'

    assert replace_function_name_in_name_with_args("""abc()""", """\"()()()\"""") == '"()()()"()'


@pytest.mark.parametrize(
    ("hostname", "expected_hostname"),
    (
        # Negative cases
        (  # New URL used
            "org-account.snowflake.com",
            "org-account.snowflake.com",
        ),
        (  # New URL used with underscore in account locator
            "org-account_locator.snowflake.com",
            "org-account-locator.snowflake.com",
        ),
        (  # No underscore in account locator
            "account.snowflake.com",
            "account.snowflake.com",
        ),
        # Positive case
        ("account_identifier.snowflake.com", "account-identifier.snowflake.com"),
    ),
)
def test_hostname_fixes(hostname, expected_hostname):
    assert (fix_hostname(hostname)) == expected_hostname


@pytest.mark.parametrize(
    ("raw_result", "expected_result"), (("1", Decimal(1)), ("1.23", Decimal("1.23")), ("-1.23e40", Decimal("-1.23e40")))
)
def test_map_result(raw_result, expected_result):
    procedure = Procedure(
        name="test_proc",
        return_type=ReturnDataType(datatype="DECFLOAT"),
        arguments=[],
        language_config=SQLFunction(),
        body="",
    )
    assert map_result(procedure, [{"test_proc": raw_result}], extract=True) == expected_result



from typing import Optional

import pytest

from snowflake.core import CreateMode
from snowflake.core.alert import Alert, AlertCollection, MinutesSchedule
from snowflake.core.tag import TagResource, TagValue

from ..base_tag_tests import BaseTagTests
from ..utils import random_string


class TestAlertTags(BaseTagTests):
    @pytest.fixture(autouse=True)
    def setup(self, alerts: AlertCollection):
        self.alerts = alerts
        alert_name = random_string(6, "alert_for_tag_tests_")
        self.alert_res = alerts.create(
            Alert(
                name=alert_name,
                condition="SELECT 1",
                action="SELECT 2",
                schedule=MinutesSchedule(minutes=1),
            ),
            mode=CreateMode.if_not_exists,
        )
        try:
            yield
        finally:
            self.alert_res.drop(if_exists=True)

    @property
    def resource_level_name(self) -> str:
        return "ALERT"

    def set_tags(
        self, tags: dict[TagResource, TagValue], if_exists: Optional[bool] = False, resource_name: Optional[str] = None
    ):
        self.alerts[resource_name or self.alert_res.name].set_tags(tags, if_exists)

    def unset_tags(
        self, tag_resources: set[TagResource], if_exists: Optional[bool] = False, resource_name: Optional[str] = None
    ):
        self.alerts[resource_name or self.alert_res.name].unset_tags(tag_resources, if_exists)

    def get_tags(
        self, with_lineage: Optional[bool] = False, resource_name: Optional[str] = None
    ) -> dict[TagResource, TagValue]:
        return self.alerts[resource_name or self.alert_res.name].get_tags(with_lineage)



#
# Copyright (c) 2012-2023 Snowflake Computing Inc. All rights reserved.
#
from contextlib import suppress

import pytest

from snowflake.core.exceptions import APIError, NotFoundError
from snowflake.core.task import Task

from ..utils import get_task_history, random_object_name


def test_execute_basic(tasks, session):
    task_name = random_object_name()
    try:
        task = tasks.create(Task(name=task_name, definition="select 1"))
        assert not get_task_history(session, task.name)
        task.execute()
        assert len(get_task_history(session, task.name)) == 1
        with pytest.raises(NotFoundError):
            tasks["not_exists"].execute()
    finally:
        with suppress(NotFoundError):
            tasks[task_name].drop()


def test_execute_retry_last(tasks, session):
    # This test will not retry a task successfully. Instead, it asserts that the retry failure error message
    # is expected.  This test makes sure the client side has sent the right request to the server.
    #
    # To test a retry from a failed task, it needs to wait some time for the task to fail.  It would be
    # time-consuming and flaky.
    task_name = random_object_name()
    try:
        task = tasks.create(Task(name=task_name, definition="select 1"))
        with pytest.raises(APIError) as exec_info:
            task.execute(retry_last=True)
        assert exec_info.value.status == 400
        assert exec_info.match("(?i)Cannot perform retry")
    finally:
        with suppress(NotFoundError):
            tasks[task_name].drop()



from collections.abc import Iterator
from typing import TYPE_CHECKING, Optional

from snowflake.core import PollingOperation
from snowflake.core._internal.telemetry import api_telemetry
from snowflake.core.account._generated.models.account import AccountModel as Account

from ._generated.api.account_api_base import AccountCollectionBase, AccountResourceBase


if TYPE_CHECKING:
    from snowflake.core import Root


class AccountCollection(AccountCollectionBase):
    """Represents the collection operations of the Snowflake Account resource.

    With this collection, you can create, iterate through, and search for an account that you have access to
    in the current context.
    """

    def __init__(self, root: "Root") -> None:
        super().__init__(root, AccountResource)

    @api_telemetry
    def create(  # type: ignore[override]
        self,
        account: Account,
    ) -> "AccountResource":
        """Create an account in Snowflake.

        Parameters
        __________
        account: Account
            The ``Account`` object, together with the ``Account``'s properties:
            name, admin_name, email, edition; admin_password, first_name, last_name, must_change_password, region_group,
            region, comment, polaris are optional.

        """
        super().create(account=account._to_model())
        return AccountResource(account.name, self)

    @api_telemetry
    def create_async(  # type: ignore[override]
        self,
        account: Account,
    ) -> PollingOperation["AccountResource"]:
        """An asynchronous version of :func:`create`.

        Refer to :class:`~snowflake.core.PollingOperation` for more information on asynchronous execution and
        the return type.
        """  # noqa: D401
        future = super().create_async(account=account._to_model())
        return PollingOperation(future, lambda _: AccountResource(account.name, self))

    @api_telemetry
    def iter(  # type: ignore[override]
        self,
        *,
        like: Optional[str] = None,
        limit: Optional[int] = None,
        history: Optional[bool] = None,
    ) -> Iterator[Account]:
        """Iterate through ``Account`` objects in Snowflake, filtering on any optional ``like`` pattern.

        Parameters
        __________
        like: str, optional
            A case-insensitive string functioning as a filter, with support for SQL
            wildcard characters (% and _).
        limit: int, optional
            Limit of the maximum number of rows returned by iter(). The default is ``None``, which behaves equivalently
            to show_limit=10000. This value must be between ``1`` and ``10000``.
        history: bool, optional
            If ``True``, includes dropped accounts that have not yet been deleted. The default is ``None``, which
            behaves equivalently to ``False``.
        """
        result = super().iter(
            like=like,
            limit=limit,
            history=history,
        )
        return (Account(**r.to_dict()) for r in result)

    @api_telemetry
    def iter_async(  # type: ignore[override]
        self,
        *,
        like: Optional[str] = None,
        limit: Optional[int] = None,
        history: Optional[bool] = None,
    ) -> PollingOperation[Iterator[Account]]:
        """An asynchronous version of :func:`iter`.

        Refer to :class:`~snowflake.core.PollingOperation` for more information on asynchronous execution and
        the return type.
        """  # noqa: D401
        future = super().iter_async(
            like=like,
            limit=limit,
            history=history,
        )
        return PollingOperation(future, lambda acc: (Account(**a.to_dict()) for a in iter(acc)))


class AccountResource(AccountResourceBase):
    """Represents a reference to a Snowflake account.

    With this account reference, you can fetch information about an account, as well
    as perform certain actions on it.
    """

    def __init__(self, name: str, collection: AccountCollection) -> None:
        super().__init__(name, collection)



import copy
import datetime

from collections.abc import Generator
from contextlib import suppress

import pytest as pytest

from snowflake.core._internal.utils import normalize_and_unquote_name, normalize_datatype
from snowflake.core.database import DatabaseResource
from snowflake.core.exceptions import NotFoundError
from snowflake.core.schema import SchemaResource
from snowflake.core.table import PrimaryKey, Table, TableColumn, TableResource, UniqueKey
from tests.integ.utils import array_equal_comparison, random_string


test_table_template = Table(
    name="<to be set>",
    columns=[
        TableColumn(
            name="c1",
            datatype="int",
            nullable=False,
            autoincrement=True,
            autoincrement_start=0,
            autoincrement_increment=1,
        ),
        TableColumn(name="c2", datatype="string"),
        TableColumn(name="c3", datatype="string", collate="FR"),
    ],
    cluster_by=["c1>1", "c2"],
    enable_schema_evolution=True,
    change_tracking=True,
    data_retention_time_in_days=1,
    max_data_extension_time_in_days=1,
    default_ddl_collation="en",
    constraints=[PrimaryKey(name="pk1", column_names=["c1"]), UniqueKey(name="uk1", column_names=["c2"])],
    comment="test table",
)


@pytest.fixture
def table_postfix() -> str:
    return random_string(10)


@pytest.fixture
def table_handle(table_postfix, tables, session) -> Generator[TableResource, None, None]:
    # Use bridge to bring handles.
    table_name = f"test_table_{table_postfix}_case_insensitive"
    test_table = copy.deepcopy(test_table_template)
    test_table.name = table_name
    test_table_handle = tables.create(test_table)
    session.sql(f"use database {tables.database.name}").collect()
    session.sql(f"use schema {tables.schema.name}").collect()
    sql_string = "insert into " + table_name + " values (1, 'random_string_1', 'random_string_2')"
    session.sql(sql_string).collect()
    sql_string = "insert into " + table_name + " values (2, 'random_string_3', 'random_string_4')"
    session.sql(sql_string).collect()
    try:
        yield test_table_handle
    finally:
        with suppress(NotFoundError):
            test_table_handle.drop()


@pytest.fixture
def table_handle_case_senstitive(table_postfix, tables, session) -> Generator[TableResource, None, None]:
    # Use bridge to bring handles.
    table_name = f'"test_table_{table_postfix}_case_sensitive"'
    test_table = copy.deepcopy(test_table_template)
    test_table.name = table_name
    test_table_handle = tables.create(test_table)
    session.sql(f"use database {tables.database.name}").collect()
    session.sql(f"use schema {tables.schema.name}").collect()
    sql_string = "insert into " + table_name + " values (1, 'random_string_1', 'random_string_2')"
    session.sql(sql_string).collect()
    sql_string = "insert into " + table_name + " values (2, 'random_string_3', 'random_string_4')"
    session.sql(sql_string).collect()
    try:
        yield test_table_handle
    finally:
        with suppress(NotFoundError):
            test_table_handle.drop()


def assert_table(
    table: Table, name: str, database: DatabaseResource, schema: SchemaResource, deep: bool = False, rows: int = 2
) -> None:
    # `Table` is fetched from the server and its attributes are checked.
    assert table.name == normalize_and_unquote_name(name)
    if deep:
        for i in range(len(table.columns)):
            assert table.columns[i].name == normalize_and_unquote_name(test_table_template.columns[i].name)
            assert normalize_datatype(table.columns[i].datatype) == normalize_datatype(
                test_table_template.columns[i].datatype
            )
            assert bool(table.columns[i].nullable) == bool(test_table_template.columns[i].nullable)
            assert table.columns[i].default == test_table_template.columns[i].default
            assert table.columns[i].autoincrement_start == test_table_template.columns[i].autoincrement_start
            assert table.columns[i].autoincrement_increment == test_table_template.columns[i].autoincrement_increment
            assert bool(table.columns[i].autoincrement) == bool(test_table_template.columns[i].autoincrement)
            assert table.columns[i].comment == test_table_template.columns[i].comment
        assert table.columns[2].collate.upper() == "FR"

        fetched_constraints = sorted(table.constraints, key=lambda x: x.name)
        original_constraints = sorted(test_table_template.constraints, key=lambda x: x.name)
        for i in range(len(fetched_constraints)):
            assert fetched_constraints[i].__class__ == original_constraints[i].__class__
            assert normalize_and_unquote_name(fetched_constraints[i].name) == normalize_and_unquote_name(
                original_constraints[i].name
            )
            assert [normalize_and_unquote_name(x) for x in fetched_constraints[i].column_names] == [
                normalize_and_unquote_name(x) for x in original_constraints[i].column_names
            ]
    else:
        assert table.columns is None
        assert table.constraints is None

    """Test behavior change between Client Bridge Vs Rest
    REST would always return these basic information
    """
    assert array_equal_comparison(table.cluster_by, test_table_template.cluster_by)
    assert table.comment == test_table_template.comment
    assert table.enable_schema_evolution is test_table_template.enable_schema_evolution
    assert table.change_tracking is test_table_template.change_tracking
    assert table.data_retention_time_in_days == test_table_template.data_retention_time_in_days
    assert table.max_data_extension_time_in_days == test_table_template.max_data_extension_time_in_days
    assert table.default_ddl_collation.upper() == test_table_template.default_ddl_collation.upper()
    assert table.automatic_clustering is True
    assert table.owner_role_type is not None
    assert table.rows == rows

    assert isinstance(table.created_on, datetime.datetime)
    assert table.dropped_on is None
    assert table.database_name == normalize_and_unquote_name(database.name)
    assert table.schema_name == normalize_and_unquote_name(schema.name)
    assert table.search_optimization is False
    assert table.search_optimization_bytes is None
    assert table.search_optimization_progress is None



import logging
import pathlib
import tempfile
import typing


tempdir = pathlib.Path(tempfile.gettempdir())


def simple_file_logging(
    path: typing.Union[str, pathlib.Path, None] = None,
    *,
    level: int = logging.DEBUG,
    format: str = "%(asctime)s - %(name)s - %(levelname)s - %(message)s",
) -> None:
    """Set up snowflake.core logging to a file.

    If path is not given the tempfile module will be used to determine the
    appropiate temporary location and a file named "snowflake_core.log"
    will be created.
    The parameter level controls what logging level is enabled in the file.
    The format parameter allows to modify the formatter output in the file.
    """
    if path is None:
        path = tempdir / "snowflake_core.log"
    logger = logging.getLogger("snowflake.core")
    # Increase log level in case the log level of
    #  the top-level logger is lower than necessary
    if not logger.isEnabledFor(level):
        logger.setLevel(level)
    fh = logging.FileHandler(path)
    fh.setLevel(level)
    fh.setFormatter(logging.Formatter(format))
    logger.addHandler(fh)



from contextlib import suppress

import pytest

from snowflake.core.exceptions import NotFoundError
from snowflake.core.schema import Schema
from snowflake.core.table import Table, TableColumn
from tests.utils import random_string


def test_swap(tables, temp_db):
    table1_name = random_string(10, "test_table_")
    table2_name = random_string(10, "test_table_")
    test_table1_handle = tables[table1_name]
    test_table2_handle = tables[table2_name]

    test_table1 = Table(name=table1_name, columns=[TableColumn(name="c1", datatype="int")])
    try:
        _ = tables.create(test_table1)
        test_table2 = Table(name=table2_name, columns=[TableColumn(name="c2", datatype="int")])
        _ = tables.create(test_table2)
        test_table1_handle.swap_with(table2_name)
        fetched_table1 = test_table1_handle.fetch()
        fetched_table2 = test_table2_handle.fetch()
        assert fetched_table1.columns[0].name == "C2"
        assert fetched_table2.columns[0].name == "C1"

        # swap with random name
        with pytest.raises(NotFoundError):
            test_table1_handle.swap_with("RANDOM")

        # swap with non-existent table but with if_exists=True
        tables["RANDOM"].swap_with(table2_name, if_exists=True)
    finally:
        with suppress(NotFoundError):
            test_table1_handle.drop()
        with suppress(NotFoundError):
            test_table2_handle.drop()


def test_swap_across_database(tables, temp_db):
    table1_name = random_string(10, "test_table_")
    table2_name = random_string(10, "test_table_")
    test_table1_handle = tables[table1_name]

    test_table1 = Table(name=table1_name, columns=[TableColumn(name="c1", datatype="int")])
    try:
        _ = tables.create(test_table1)

        schema_name = random_string(10, "test_create_clone_across_schema_")
        temp_db.schemas.create(Schema(name=schema_name))

        test_table2 = Table(name=table2_name, columns=[TableColumn(name="c2", datatype="int")])
        _ = temp_db.schemas[schema_name].tables.create(test_table2)
        test_table1_handle.swap_with(table2_name, target_database=temp_db.name, target_schema=schema_name)
        fetched_table1 = test_table1_handle.fetch()
        test_table2_handle = temp_db.schemas[schema_name].tables[table2_name]
        fetched_table2 = test_table2_handle.fetch()
        assert fetched_table1.columns[0].name == "C2"
        assert fetched_table2.columns[0].name == "C1"
    finally:
        with suppress(NotFoundError):
            test_table1_handle.drop()
        with suppress(NotFoundError):
            test_table2_handle.drop()



from __future__ import absolute_import

# flake8: noqa

# import apis into api package
from snowflake.core.cortex.search_service._generated.api.cortex_search_service_api import CortexSearchServiceApi

__all__ = [
    "CortexSearchServiceApi",
]



import copy

import pytest

from tests.integ.utils import random_string

from .conftest import test_network_rule_template


@pytest.fixture(scope="module")
def network_rules_extended(network_rules):
    names_list = []

    for _ in range(5):
        names_list.append(random_string(10, "test_network_rule_iter_a_"))

    for _ in range(7):
        names_list.append(random_string(10, "test_network_rule_iter_b_"))

    for _ in range(3):
        names_list.append(random_string(10, "test_network_rule_iter_c_"))

    try:
        for name in names_list:
            network_rule = copy.deepcopy(test_network_rule_template)
            network_rule.name = name
            network_rules.create(network_rule)

        yield network_rules
    finally:
        for name in names_list:
            network_rules[name].drop(if_exists=True)


def test_iter_raw(network_rules_extended):
    assert len(list(network_rules_extended.iter())) >= 15


def test_iter_like(network_rules_extended):
    assert len(list(network_rules_extended.iter(like="test_network_rule_iter_a_%"))) == 5
    assert len(list(network_rules_extended.iter(like="test_network_rule_iter_b_%"))) == 7
    assert len(list(network_rules_extended.iter(like="test_network_rule_iter_c_%"))) == 3
    assert len(list(network_rules_extended.iter(like="TEST_NETWORK_RULE_ITER_C_%"))) == 3
    assert len(list(network_rules_extended.iter(like="nonexistent_pattern_%"))) == 0


def test_iter_limit(network_rules_extended):
    assert len(list(network_rules_extended.iter(limit=2))) == 2


def test_iter_starts_with(network_rules_extended):
    assert len(list(network_rules_extended.iter(starts_with="test_network_rule_iter_a_".upper()))) == 5
    assert len(list(network_rules_extended.iter(starts_with="test_network_rule_iter_a_"))) == 0
    assert len(list(network_rules_extended.iter(starts_with="test_network_rule_iter_d_".upper()))) == 0


def test_iter_from_name(network_rules_extended):
    # The limit keyword is required for the from keyword to function, limit=20 was chosen arbitrarily
    # as it does not affect the test
    rules_from_b = list(network_rules_extended.iter(limit=20, from_name="test_network_rule_iter_b_".upper()))
    assert len(rules_from_b) == 10



import time

from contextlib import suppress
from datetime import datetime, timezone

import pytest

from snowflake.core._common import CreateMode
from snowflake.core.exceptions import ConflictError, NotFoundError
from snowflake.core.stream import (
    PointOfTimeOffset,
    PointOfTimeStatement,
    PointOfTimeStream,
    PointOfTimeTimestamp,
    Stream,
    StreamSourceStage,
    StreamSourceTable,
    StreamSourceView,
)
from tests.utils import random_string


def create_stream(name, stream_source, streams):
    stream = Stream(name=name, stream_source=stream_source)

    created_stream = streams.create(stream)

    stream_handle = created_stream.fetch()
    assert stream_handle.name.upper() == name.upper()
    assert stream_handle.stream_source is not None
    assert type(stream_handle.stream_source) is type(stream_source)
    return stream_handle


@pytest.mark.min_sf_ver("8.35.0")
@pytest.mark.parametrize("pot", (None, PointOfTimeOffset(reference="before", offset="-1")))
def test_create_by_table(streams, src_temp_table, pot):
    stream_name = random_string(10, "test_stream_by_table_")
    try:
        if pot is not None:
            time.sleep(1)
        create_stream(
            stream_name,
            StreamSourceTable(
                point_of_time=pot, name=src_temp_table.name, append_only=True, show_initial_rows=False, comment="asdf"
            ),
            streams,
        )
    finally:
        with suppress(NotFoundError):
            streams[stream_name].drop()


def test_create_by_table_stream_pot(streams, src_temp_table):
    reference_stream_name = random_string(10, "test_reference_stream_")
    stream_name = random_string(10, "test_pot_stream_")
    pot = PointOfTimeStream(reference="at", stream=reference_stream_name)

    try:
        create_stream(
            reference_stream_name,
            StreamSourceTable(name=src_temp_table.name, append_only=True, show_initial_rows=False),
            streams,
        )
        time.sleep(1)
        create_stream(
            stream_name,
            StreamSourceTable(point_of_time=pot, name=src_temp_table.name, append_only=True, show_initial_rows=False),
            streams,
        )
    finally:
        with suppress(NotFoundError):
            streams[reference_stream_name].drop()
            streams[stream_name].drop()


def test_create_by_table_timestamp_pot(streams, src_temp_table):
    stream_name = random_string(10, "test_pot_timestamp_")
    time.sleep(1)
    epoch = int(datetime.now(timezone.utc).timestamp())
    time.sleep(1)
    pot = PointOfTimeTimestamp(reference="at", timestamp=f"TO_TIMESTAMP_TZ({epoch})")
    try:
        create_stream(
            stream_name,
            StreamSourceTable(point_of_time=pot, name=src_temp_table.name, append_only=True, show_initial_rows=False),
            streams,
        )
    finally:
        with suppress(NotFoundError):
            streams[stream_name].drop()


def test_create_by_table_statement_pot(streams, src_temp_table, connection):
    stream_name = random_string(10, "test_pot_statement_")
    time.sleep(1)
    with connection.cursor() as cursor:
        query_id = cursor.execute("SELECT 1").sfqid
    time.sleep(1)
    pot = PointOfTimeStatement(reference="at", statement=query_id)
    try:
        create_stream(
            stream_name,
            StreamSourceTable(point_of_time=pot, name=src_temp_table.name, append_only=True, show_initial_rows=False),
            streams,
        )
    finally:
        with suppress(NotFoundError):
            streams[stream_name].drop()


@pytest.mark.min_sf_ver("8.35.0")
@pytest.mark.parametrize("pot", (None, PointOfTimeOffset(reference="before", offset="-1")))
def test_create_by_view(streams, src_temp_view, pot):
    stream_name = random_string(10, "test_stream_by_view_")

    try:
        if pot is not None:
            time.sleep(1)
        create_stream(stream_name, StreamSourceView(point_of_time=pot, name=src_temp_view.name), streams)
    finally:
        with suppress(NotFoundError):
            streams[stream_name].drop()


@pytest.mark.min_sf_ver("8.35.0")
@pytest.mark.parametrize("pot", (None, PointOfTimeOffset(reference="before", offset="-1")))
def test_create_by_directory_table(streams, temp_directory_table, pot):
    stream_name = random_string(10, "test_stream_by_stage_")

    try:
        if pot is not None:
            time.sleep(1)
        create_stream(stream_name, StreamSourceStage(point_of_time=pot, name=temp_directory_table.name), streams)
    finally:
        with suppress(NotFoundError):
            streams[stream_name].drop()


@pytest.mark.min_sf_ver("8.35.0")
@pytest.mark.parametrize("pot", (None, PointOfTimeOffset(reference="before", offset="-1")))
def test_create_by_view_with_multiple_base_tables(streams, src_temp_view_with_multiple_base_tables, pot):
    stream_name = random_string(10, "test_stream_by_view_with_multiple_base_tables_")

    view_handle, table_a_name, table_b_name = src_temp_view_with_multiple_base_tables
    try:
        if pot is not None:
            time.sleep(1)
        stream_handle = create_stream(stream_name, StreamSourceView(point_of_time=pot, name=view_handle.name), streams)

        assert isinstance(stream_handle.stream_source, StreamSourceView)
        assert sorted([i.split(".")[-1] for i in stream_handle.stream_source.base_tables]) == sorted(
            [table_a_name.upper(), table_b_name.upper()]
        )
    finally:
        with suppress(NotFoundError):
            streams[stream_name].drop()


@pytest.mark.min_sf_ver("8.35.0")
def test_create_and_update(streams, src_temp_table):
    stream_name = random_string(10, "test_stream_by_table_")

    try:
        stream_handle = streams.create(
            Stream(name=stream_name, stream_source=StreamSourceTable(name=src_temp_table.name))
        )

        streams.create(
            Stream(name=stream_name, stream_source=StreamSourceTable(name=src_temp_table.name, copy_grants=True)),
            mode=CreateMode.or_replace,
        )

        with pytest.raises(ConflictError):
            streams.create(Stream(name=stream_name, stream_source=StreamSourceTable(name=src_temp_table.name)))

        streams.create(stream_handle.fetch(), mode=CreateMode.or_replace)
    finally:
        with suppress(NotFoundError):
            streams[stream_name].drop()



from unittest import mock

import pytest

from snowflake.core import PollingOperation
from snowflake.core.cortex.embed_service import CortexEmbedService

from ...utils import BASE_URL, extra_params


API_CLIENT_REQUEST = "snowflake.core._generated.api_client.ApiClient.request"


@pytest.fixture
def embed_service(fake_root):
    return CortexEmbedService(fake_root)


def test_embed(fake_root, embed_service):
    args = (fake_root, "POST", BASE_URL + "/cortex/inference:embed")
    kwargs = extra_params(body={"model": "my_model", "text": ["xyz"], "provisioned_throughput_id": None})

    with mock.patch(API_CLIENT_REQUEST) as mocked_request:
        op = embed_service.embed_async("my_model", ["xyz"])
        assert isinstance(op, PollingOperation)
        op.result()
    mocked_request.assert_called_once_with(*args, **kwargs)



from unittest import mock

import pytest

from snowflake.core import PollingOperation
from snowflake.core.tag import TagValue
from snowflake.core.warehouse import Warehouse, WarehouseCollection, WarehouseResource
from snowflake.core.warehouse._generated import TagAssignment, TagReference

from ...utils import BASE_URL, extra_params, mock_http_response


API_CLIENT_REQUEST = "snowflake.core._generated.api_client.ApiClient.request"


@pytest.fixture
def warehouses(fake_root):
    return WarehouseCollection(fake_root)


@pytest.fixture
def warehouse(warehouses):
    return warehouses["my_wh"]


def test_create_warehouse(fake_root, warehouses):
    args = (fake_root, "POST", BASE_URL + "/warehouses?createMode=errorIfExists")
    kwargs = extra_params(query_params=[("createMode", "errorIfExists")], body={"name": "my_wh"})

    with mock.patch(API_CLIENT_REQUEST) as mocked_request:
        warehouse_res = warehouses.create(Warehouse(name="my_wh"))
        assert isinstance(warehouse_res, WarehouseResource)
        assert warehouse_res.name == "my_wh"
    mocked_request.assert_called_once_with(*args, **kwargs)

    with mock.patch(API_CLIENT_REQUEST) as mocked_request:
        op = warehouses.create_async(Warehouse(name="my_wh"))
        assert isinstance(op, PollingOperation)
        wh_res = op.result()
        assert wh_res.name == "my_wh"
    mocked_request.assert_called_once_with(*args, **kwargs)


def test_iter_warehouse(fake_root, warehouses):
    args = (fake_root, "GET", BASE_URL + "/warehouses")
    kwargs = extra_params()

    with mock.patch(API_CLIENT_REQUEST) as mocked_request:
        mocked_request.return_value = mock_http_response()
        warehouses.iter()
    mocked_request.assert_called_once_with(*args, **kwargs)

    with mock.patch(API_CLIENT_REQUEST) as mocked_request:
        mocked_request.return_value = mock_http_response()
        op = warehouses.iter_async()
        assert isinstance(op, PollingOperation)
        it = op.result()
        assert list(it) == []
    mocked_request.assert_called_once_with(*args, **kwargs)


def test_create_or_alter_warehouse(fake_root, warehouse):
    args = (fake_root, "PUT", BASE_URL + "/warehouses/my_wh")
    kwargs = extra_params(body={"name": "my_wh"})

    with mock.patch(API_CLIENT_REQUEST) as mocked_request:
        warehouse.create_or_alter(Warehouse(name="my_wh"))
    mocked_request.assert_called_once_with(*args, **kwargs)

    with mock.patch(API_CLIENT_REQUEST) as mocked_request:
        op = warehouse.create_or_alter_async(Warehouse(name="my_wh"))
        assert isinstance(op, PollingOperation)
        op.result()
    mocked_request.assert_called_once_with(*args, **kwargs)


def test_suspend_warehouse(fake_root, warehouse):
    args = (fake_root, "POST", BASE_URL + "/warehouses/my_wh:suspend")
    kwargs = extra_params()

    with mock.patch(API_CLIENT_REQUEST) as mocked_request:
        warehouse.suspend()
    mocked_request.assert_called_once_with(*args, **kwargs)

    with mock.patch(API_CLIENT_REQUEST) as mocked_request:
        op = warehouse.suspend_async()
        assert isinstance(op, PollingOperation)
        op.result()
    mocked_request.assert_called_once_with(*args, **kwargs)


def test_resume_warehouse(fake_root, warehouse):
    args = (fake_root, "POST", BASE_URL + "/warehouses/my_wh:resume")
    kwargs = extra_params()

    with mock.patch(API_CLIENT_REQUEST) as mocked_request:
        warehouse.resume()
    mocked_request.assert_called_once_with(*args, **kwargs)

    with mock.patch(API_CLIENT_REQUEST) as mocked_request:
        op = warehouse.resume_async()
        assert isinstance(op, PollingOperation)
        op.result()
    mocked_request.assert_called_once_with(*args, **kwargs)


def test_fetch_warehouse(fake_root, warehouse):
    from snowflake.core.warehouse._generated.models import Warehouse as WarehouseModel

    model = WarehouseModel(name="my_wh")
    args = (fake_root, "GET", BASE_URL + "/warehouses/my_wh")
    kwargs = extra_params()

    with mock.patch(API_CLIENT_REQUEST) as mocked_request:
        mocked_request.return_value = mock_http_response(model.to_json())
        warehouse.fetch()
    mocked_request.assert_called_once_with(*args, **kwargs)

    with mock.patch(API_CLIENT_REQUEST) as mocked_request:
        mocked_request.return_value = mock_http_response(model.to_json())
        op = warehouse.fetch_async()
        assert isinstance(op, PollingOperation)
        tab = op.result()
        assert tab.to_dict() == Warehouse(name="my_wh").to_dict()
    mocked_request.assert_called_once_with(*args, **kwargs)


def test_drop_warehouse(fake_root, warehouse):
    args = (fake_root, "DELETE", BASE_URL + "/warehouses/my_wh")
    kwargs = extra_params()

    with mock.patch(API_CLIENT_REQUEST) as mocked_request:
        warehouse.drop()
    mocked_request.assert_called_once_with(*args, **kwargs)

    with mock.patch(API_CLIENT_REQUEST) as mocked_request:
        op = warehouse.drop_async()
        assert isinstance(op, PollingOperation)
        op.result()
    mocked_request.assert_called_once_with(*args, **kwargs)


def test_rename_warehouse(fake_root, warehouse):
    args = (fake_root, "POST", BASE_URL + "/warehouses/my_wh:rename")
    kwargs = extra_params(body={"name": "new_wh"})

    with mock.patch(API_CLIENT_REQUEST) as mocked_request:
        warehouse.rename("new_wh")
    mocked_request.assert_called_once_with(*args, **kwargs)

    with mock.patch(API_CLIENT_REQUEST) as mocked_request:
        op = warehouse.rename_async("new_wh")
        assert isinstance(op, PollingOperation)
        op.result()
    mocked_request.assert_called_once_with(*args, **kwargs)


def test_abort_all_queries(fake_root, warehouse):
    args = (fake_root, "POST", BASE_URL + "/warehouses/my_wh:abort")
    kwargs = extra_params()

    with mock.patch(API_CLIENT_REQUEST) as mocked_request:
        warehouse.abort_all_queries()
    mocked_request.assert_called_once_with(*args, **kwargs)

    with mock.patch(API_CLIENT_REQUEST) as mocked_request:
        op = warehouse.abort_all_queries_async()
        assert isinstance(op, PollingOperation)
        op.result()
    mocked_request.assert_called_once_with(*args, **kwargs)


def test_use_warehouse(fake_root, warehouse):
    args = (fake_root, "POST", BASE_URL + "/warehouses/my_wh:use")
    kwargs = extra_params()

    with mock.patch(API_CLIENT_REQUEST) as mocked_request:
        warehouse.use_warehouse()
    mocked_request.assert_called_once_with(*args, **kwargs)

    with mock.patch(API_CLIENT_REQUEST) as mocked_request:
        op = warehouse.use_warehouse_async()
        assert isinstance(op, PollingOperation)
        op.result()
    mocked_request.assert_called_once_with(*args, **kwargs)


def test_set_tags(fake_root, warehouse, tag):
    args = (fake_root, "POST", BASE_URL + "/warehouses/my_wh:set-tags")
    tags = {tag: TagValue(value="value")}
    kwargs = extra_params(
        body=[
            TagAssignment(
                tag_value=v.value, tag_name=k.name, tag_schema=k.schema.name, tag_database=k.database.name
            ).to_dict()
            for k, v in tags.items()
        ]
    )

    with mock.patch(API_CLIENT_REQUEST) as mocked_request:
        warehouse.set_tags(tags)
    mocked_request.assert_called_once_with(*args, **kwargs)

    with mock.patch(API_CLIENT_REQUEST) as mocked_request:
        op = warehouse.set_tags_async(tags)
        assert isinstance(op, PollingOperation)
        op.result()
    mocked_request.assert_called_once_with(*args, **kwargs)


def test_unset_tags(fake_root, warehouse, tag):
    args = (fake_root, "POST", BASE_URL + "/warehouses/my_wh:unset-tags")
    tag_resources = {tag}
    kwargs = extra_params(
        body=[
            TagReference(
                tag_name=tag_res.name, tag_schema=tag_res.schema.name, tag_database=tag_res.database.name
            ).to_dict()
            for tag_res in tag_resources
        ]
    )

    with mock.patch(API_CLIENT_REQUEST) as mocked_request:
        warehouse.unset_tags(tag_resources)
    mocked_request.assert_called_once_with(*args, **kwargs)

    with mock.patch(API_CLIENT_REQUEST) as mocked_request:
        op = warehouse.unset_tags_async(tag_resources)
        assert isinstance(op, PollingOperation)
        op.result()
    mocked_request.assert_called_once_with(*args, **kwargs)


def test_get_tags(fake_root, warehouse):
    args = (fake_root, "GET", BASE_URL + "/warehouses/my_wh:get-tags")
    kwargs = extra_params()

    with mock.patch(API_CLIENT_REQUEST) as mocked_request:
        mocked_request.return_value = mock_http_response()
        assert warehouse.get_tags() == {}
    mocked_request.assert_called_once_with(*args, **kwargs)

    with mock.patch(API_CLIENT_REQUEST) as mocked_request:
        mocked_request.return_value = mock_http_response()
        op = warehouse.get_tags_async()
        assert isinstance(op, PollingOperation)
        assert op.result() == {}
    mocked_request.assert_called_once_with(*args, **kwargs)



import pytest

from snowflake.core.streamlit import (
    AddVersionFromGitStreamlitRequest,
    Streamlit,
    StreamlitVersionForGit,
)
from tests.integ.utils import random_string


@pytest.mark.min_sf_ver("9.38.0")
def test_add_version_from_git_streamlit(
    streamlits, streamlit_stage_with_file, warehouse, streamlit_main_file, git_repository
):
    if not git_repository.get("commit_hash") or not git_repository.get("remote_url"):
        pytest.skip(
            "Git not available or no remote repository. Set TEST_GIT_REPO_URL, GITHUB_TOKEN, or use GitHub CLI."
        )

    name = random_string(8, "test_add_version_from_git_streamlit_")

    st = Streamlit(
        name=name,
        query_warehouse=warehouse.name,
        source_location=f"@{streamlit_stage_with_file.name}",
        main_file=streamlit_main_file,
    )

    ref = streamlits.create(st)
    try:
        version_name = random_string(8, "git_version_")
        version = StreamlitVersionForGit(
            name=version_name,
            comment="Test Git version for integration testing",
        )

        # Use remote URL with commit hash
        git_ref = f"{git_repository['remote_url']}@{git_repository['commit_hash']}"

        add_version_from_git_request = AddVersionFromGitStreamlitRequest(
            version=version,
            git_ref=git_ref,
        )

        ref.add_version_from_git(add_version_from_git_request)

        fetched = ref.fetch()
        assert fetched.name.upper() == name.upper()

        assert fetched.last_version_details is not None
        assert fetched.last_version_details.name is not None
        assert fetched.last_version_details.alias.upper() == version_name.upper()

    finally:
        ref.drop()



"""
Cortex Analyst API.

The Snowflake Cortex Analyst API is a REST API that allows end user to chat with their data leveraging semantic models to generate SQL queries.  # noqa: E501

The version of the OpenAPI document: 0.0.1
Contact: support@snowflake.com
Generated by: https://openapi-generator.tech

Do not edit this file manually.
"""

from __future__ import annotations

import json
import pprint
import re  # noqa: F401

from typing import Any, Dict, Optional

from pydantic import BaseModel, ConfigDict, StrictStr

from snowflake.core.cortex.analyst_service._generated.models.semantic_model_change_operation import (
    SemanticModelChangeOperation,
)


class SemanticModelChange(BaseModel):
    """A model object representing the SemanticModelChange resource.

    Constructs an object of type SemanticModelChange with the provided properties.

    Parameters
    __________
    operation : SemanticModelChangeOperation

    path : str
        Path to the node in the semantic object to change
    value : object, optional
        One-of value. A map with exactly one key set corresponding to the semantic model object or primitive.
    """

    operation: SemanticModelChangeOperation

    path: StrictStr

    value: Optional[Dict[str, Any]] = None

    __properties = ["operation", "path", "value"]

    model_config = ConfigDict(
        validate_by_name=True,
        validate_assignment=True,
    )

    def to_str(self) -> str:
        """Returns the string representation of the model using alias."""
        return pprint.pformat(self.dict(by_alias=True))

    def to_json(self) -> str:
        """Returns the JSON representation of the model using alias."""
        return json.dumps(self.to_dict())

    @classmethod
    def from_json(cls, json_str: str) -> SemanticModelChange:
        """Create an instance of SemanticModelChange from a JSON string."""
        return cls.from_dict(json.loads(json_str))

    def to_dict(
        self,
        hide_readonly_properties: bool = False,
    ) -> dict[str, Any]:
        """Returns the dictionary representation of the model using alias."""
        exclude_properties = set()

        if hide_readonly_properties:
            exclude_properties.update({})

        _dict = self.model_dump(serialize_as_any=True, by_alias=True, exclude=exclude_properties, exclude_none=True)

        return _dict

    def to_dict_without_readonly_properties(self) -> dict[str, Any]:
        """Return the dictionary representation of the model without readonly properties."""
        return self.to_dict(hide_readonly_properties=True)

    @classmethod
    def from_dict(cls, obj: dict) -> SemanticModelChange:
        """Create an instance of SemanticModelChange from a dict."""
        if obj is None:
            return None

        if type(obj) is not dict:
            return SemanticModelChange.model_validate(obj)

        _obj = SemanticModelChange.model_validate(
            {
                "operation": obj.get("operation"),
                "path": obj.get("path"),
                "value": obj.get("value"),
            }
        )

        return _obj


class SemanticModelChangeModel:
    def __init__(
        self,
        operation: SemanticModelChangeOperation,
        path: str,
        # optional properties
        value: Optional[object] = None,
    ):
        """A model object representing the SemanticModelChange resource.

        Constructs an object of type SemanticModelChange with the provided properties.

        Parameters
        __________
        operation : SemanticModelChangeOperation

        path : str
            Path to the node in the semantic object to change
        value : object, optional
            One-of value. A map with exactly one key set corresponding to the semantic model object or primitive.
        """
        self.operation = operation
        self.path = path
        self.value = value

    __properties = ["operation", "path", "value"]

    def __repr__(self) -> str:
        return repr(self._to_model())

    def _to_model(self):
        return SemanticModelChange(
            operation=self.operation,
            path=self.path,
            value=self.value,
        )

    @classmethod
    def _from_model(cls, model) -> SemanticModelChangeModel:
        return SemanticModelChangeModel(
            operation=model.operation,
            path=model.path,
            value=model.value,
        )

    def to_dict(self):
        """Create a dictionary of the properties from a SemanticModelChange.

        This method constructs a dictionary with the key-value entries corresponding to the properties of the SemanticModelChange object.

        Returns
        _______
        dict
            A dictionary object created using the input model.
        """
        return self._to_model().to_dict()

    @classmethod
    def from_dict(cls, obj: dict) -> SemanticModelChangeModel:
        """Create an instance of SemanticModelChange from a dict.

        This method constructs a SemanticModelChange object from a dictionary with the key-value pairs of its properties.

        Parameters
        ----------
        obj : dict
            A dictionary whose keys and values correspond to the properties of the resource object.

        Returns
        _______
        SemanticModelChange
            A SemanticModelChange object created using the input dictionary; this will fail if the required properties are missing.
        """
        return cls._from_model(SemanticModelChange.from_dict(obj))


SemanticModelChange._model_class = SemanticModelChangeModel



"""
Snowflake View API.

The Snowflake View API is a REST API that you can use to access, update, and perform certain actions on View resource in a Snowflake database.  # noqa: E501

The version of the OpenAPI document: 0.0.1
Contact: support@snowflake.com
Generated by: https://openapi-generator.tech

Do not edit this file manually.
"""

from __future__ import annotations

import json
import pprint
import re  # noqa: F401

from typing import Any, Optional

from pydantic import BaseModel, ConfigDict, StrictStr


class ErrorResponse(BaseModel):
    """A model object representing the ErrorResponse resource.

    Constructs an object of type ErrorResponse with the provided properties.

    Parameters
    __________
    message : str, optional
        Error message returned by the server
    code : str, optional
        Error code.
    error_code : str, optional
        Error code, same as `code` above. This property has been deprecated and will be removed in a future release, but is temporarily supported for for short-term backward compatibility.
    request_id : str, optional
        Unique request ID.
    """

    message: Optional[StrictStr] = None

    code: Optional[StrictStr] = None

    error_code: Optional[StrictStr] = None

    request_id: Optional[StrictStr] = None

    __properties = ["message", "code", "error_code", "request_id"]

    model_config = ConfigDict(
        validate_by_name=True,
        validate_assignment=True,
    )

    def to_str(self) -> str:
        """Returns the string representation of the model using alias."""
        return pprint.pformat(self.dict(by_alias=True))

    def to_json(self) -> str:
        """Returns the JSON representation of the model using alias."""
        return json.dumps(self.to_dict())

    @classmethod
    def from_json(cls, json_str: str) -> ErrorResponse:
        """Create an instance of ErrorResponse from a JSON string."""
        return cls.from_dict(json.loads(json_str))

    def to_dict(
        self,
        hide_readonly_properties: bool = False,
    ) -> dict[str, Any]:
        """Returns the dictionary representation of the model using alias."""
        exclude_properties = set()

        if hide_readonly_properties:
            exclude_properties.update({})

        _dict = self.model_dump(serialize_as_any=True, by_alias=True, exclude=exclude_properties, exclude_none=True)

        return _dict

    def to_dict_without_readonly_properties(self) -> dict[str, Any]:
        """Return the dictionary representation of the model without readonly properties."""
        return self.to_dict(hide_readonly_properties=True)

    @classmethod
    def from_dict(cls, obj: dict) -> ErrorResponse:
        """Create an instance of ErrorResponse from a dict."""
        if obj is None:
            return None

        if type(obj) is not dict:
            return ErrorResponse.model_validate(obj)

        _obj = ErrorResponse.model_validate(
            {
                "message": obj.get("message"),
                "code": obj.get("code"),
                "error_code": obj.get("error_code"),
                "request_id": obj.get("request_id"),
            }
        )

        return _obj


class ErrorResponseModel:
    def __init__(
        self,  # optional properties
        message: Optional[str] = None,
        code: Optional[str] = None,
        error_code: Optional[str] = None,
        request_id: Optional[str] = None,
    ):
        """A model object representing the ErrorResponse resource.

        Constructs an object of type ErrorResponse with the provided properties.

        Parameters
        __________
        message : str, optional
            Error message returned by the server
        code : str, optional
            Error code.
        error_code : str, optional
            Error code, same as `code` above. This property has been deprecated and will be removed in a future release, but is temporarily supported for for short-term backward compatibility.
        request_id : str, optional
            Unique request ID.
        """
        self.message = message
        self.code = code
        self.error_code = error_code
        self.request_id = request_id

    __properties = ["message", "code", "error_code", "request_id"]

    def __repr__(self) -> str:
        return repr(self._to_model())

    def _to_model(self):
        return ErrorResponse(
            message=self.message,
            code=self.code,
            error_code=self.error_code,
            request_id=self.request_id,
        )

    @classmethod
    def _from_model(cls, model) -> ErrorResponseModel:
        return ErrorResponseModel(
            message=model.message,
            code=model.code,
            error_code=model.error_code,
            request_id=model.request_id,
        )

    def to_dict(self):
        """Create a dictionary of the properties from a ErrorResponse.

        This method constructs a dictionary with the key-value entries corresponding to the properties of the ErrorResponse object.

        Returns
        _______
        dict
            A dictionary object created using the input model.
        """
        return self._to_model().to_dict()

    @classmethod
    def from_dict(cls, obj: dict) -> ErrorResponseModel:
        """Create an instance of ErrorResponse from a dict.

        This method constructs a ErrorResponse object from a dictionary with the key-value pairs of its properties.

        Parameters
        ----------
        obj : dict
            A dictionary whose keys and values correspond to the properties of the resource object.

        Returns
        _______
        ErrorResponse
            A ErrorResponse object created using the input dictionary; this will fail if the required properties are missing.
        """
        return cls._from_model(ErrorResponse.from_dict(obj))


ErrorResponse._model_class = ErrorResponseModel



from __future__ import absolute_import

# flake8: noqa

# import apis into api package
from snowflake.core.compute_pool._generated.api.compute_pool_api import ComputePoolApi

__all__ = [
    "ComputePoolApi",
]



import copy

import pytest as pytest

from snowflake.core.exceptions import NotFoundError
from tests.integ.utils import random_string

from .conftest import test_view_template


@pytest.mark.min_sf_ver("8.33.0")
def test_drop(views, temp_table):
    prefix = "test_view_"
    pre_create_count = len(list(views.iter(like=prefix + "%%")))
    view_name = random_string(10, prefix)
    view_handle = views[view_name]
    test_view = copy.deepcopy(test_view_template)
    test_view.name = view_name
    test_view.query = f"SELECT * FROM {temp_table.name}"
    views.create(test_view)
    created_count = len(list(views.iter(like=prefix + "%%")))
    view_handle.drop()
    after_drop_count = len(list(views.iter(like=prefix + "%%")))

    with pytest.raises(NotFoundError):
        view_handle.drop(if_exists=False)

    assert pre_create_count + 1 == created_count == after_drop_count + 1

    views.create(test_view)
    created_count = len(list(views.iter(like=prefix + "%%")))
    view_handle.drop(if_exists=True)
    after_drop_count = len(list(views.iter(like=prefix + "%%")))
    assert pre_create_count + 1 == created_count == after_drop_count + 1



"""
Cortex Analyst API.

The Snowflake Cortex Analyst API is a REST API that allows end user to chat with their data leveraging semantic models to generate SQL queries.  # noqa: E501

The version of the OpenAPI document: 0.0.1
Contact: support@snowflake.com
Generated by: https://openapi-generator.tech

Do not edit this file manually.
"""

from __future__ import annotations

import json
import pprint
import re  # noqa: F401

from typing import Any, Optional

from pydantic import ConfigDict, StrictStr

from snowflake.core.cortex.analyst_service._generated.models.message_content_delta import (
    MessageContentDelta,
)


class MessageContentDeltaTextObject(MessageContentDelta):
    """A model object representing the MessageContentDeltaTextObject resource.

    Constructs an object of type MessageContentDeltaTextObject with the provided properties.

    Parameters
    __________
    text_delta : str
        The delta of the text content, clients should concatenate all deltas for the same index

    index : int, optional
        The index of the content array this delta object represents
    """

    text_delta: StrictStr

    __properties = ["type", "index", "text_delta"]

    model_config = ConfigDict(
        validate_by_name=True,
        validate_assignment=True,
    )

    def to_str(self) -> str:
        """Returns the string representation of the model using alias."""
        return pprint.pformat(self.dict(by_alias=True))

    def to_json(self) -> str:
        """Returns the JSON representation of the model using alias."""
        return json.dumps(self.to_dict())

    @classmethod
    def from_json(cls, json_str: str) -> MessageContentDeltaTextObject:
        """Create an instance of MessageContentDeltaTextObject from a JSON string."""
        return cls.from_dict(json.loads(json_str))

    def to_dict(
        self,
        hide_readonly_properties: bool = False,
    ) -> dict[str, Any]:
        """Returns the dictionary representation of the model using alias."""
        exclude_properties = set()

        if hide_readonly_properties:
            exclude_properties.update({})

        _dict = self.model_dump(serialize_as_any=True, by_alias=True, exclude=exclude_properties, exclude_none=True)

        _dict["type"] = MessageContentDelta.get_child_model_discriminator_value("MessageContentDeltaTextObject")

        return _dict

    def to_dict_without_readonly_properties(self) -> dict[str, Any]:
        """Return the dictionary representation of the model without readonly properties."""
        return self.to_dict(hide_readonly_properties=True)

    @classmethod
    def from_dict(cls, obj: dict) -> MessageContentDeltaTextObject:
        """Create an instance of MessageContentDeltaTextObject from a dict."""
        if obj is None:
            return None

        if type(obj) is not dict:
            return MessageContentDeltaTextObject.model_validate(obj)

        _obj = MessageContentDeltaTextObject.model_validate(
            {
                "index": obj.get("index"),
                "text_delta": obj.get("text_delta"),
            }
        )

        return _obj


class MessageContentDeltaTextObjectModel(MessageContentDelta):
    def __init__(
        self,
        text_delta: str,
        # optional properties
        index: Optional[int] = None,
    ):
        """A model object representing the MessageContentDeltaTextObject resource.

        Constructs an object of type MessageContentDeltaTextObject with the provided properties.

        Parameters
        __________
        text_delta : str
            The delta of the text content, clients should concatenate all deltas for the same index

        index : int, optional
            The index of the content array this delta object represents
        """
        super().__init__(
            index=index,
        )
        self.text_delta = text_delta

    __properties = ["type", "index", "text_delta"]

    def __repr__(self) -> str:
        return repr(self._to_model())

    def _to_model(self):
        return MessageContentDeltaTextObject(
            index=self.index,
            text_delta=self.text_delta,
        )

    @classmethod
    def _from_model(cls, model) -> MessageContentDeltaTextObjectModel:
        return MessageContentDeltaTextObjectModel(
            index=model.index,
            text_delta=model.text_delta,
        )

    def to_dict(self):
        """Create a dictionary of the properties from a MessageContentDeltaTextObject.

        This method constructs a dictionary with the key-value entries corresponding to the properties of the MessageContentDeltaTextObject object.

        Returns
        _______
        dict
            A dictionary object created using the input model.
        """
        return self._to_model().to_dict()

    @classmethod
    def from_dict(cls, obj: dict) -> MessageContentDeltaTextObjectModel:
        """Create an instance of MessageContentDeltaTextObject from a dict.

        This method constructs a MessageContentDeltaTextObject object from a dictionary with the key-value pairs of its properties.

        Parameters
        ----------
        obj : dict
            A dictionary whose keys and values correspond to the properties of the resource object.

        Returns
        _______
        MessageContentDeltaTextObject
            A MessageContentDeltaTextObject object created using the input dictionary; this will fail if the required properties are missing.
        """
        return cls._from_model(MessageContentDeltaTextObject.from_dict(obj))


MessageContentDeltaTextObject._model_class = MessageContentDeltaTextObjectModel



import logging

from typing import TYPE_CHECKING, Optional, Union

from snowflake.connector import SnowflakeConnection

from ._common import TokenType
from ._internal.root_configuration import RootConfiguration
from ._root import Root
from ._utils import fix_hostname as _fix_hostname
from .exceptions import SnowflakePythonError


if TYPE_CHECKING:
    from snowflake.snowpark import Session

logger = logging.getLogger(__name__)


class RESTConnection:
    """Connection to the GS REST endpoint.

    Lightweight alternative to a full-blown SnowflakeConnection for use in multi-tenant services like Spark Connect.
    """

    def __init__(
        self,
        host: str,
        port: int,
        token_type: TokenType,
        token: str,
        external_session_id: Optional[str] = None,
        protocol: str = "https",
    ) -> None:
        self.host = host
        self.port = port
        self._token_type = token_type
        self._session_token = token
        # Only applicable when token_type is EXTERNAL_SESSION_WITH_PAT. Set to Spark session ID
        self.external_session_id = external_session_id
        self.protocol = protocol

    @property
    def url(self) -> str:
        return f"{self.protocol}://{self.host}:{self.port}"

    @property
    def session_token(self) -> str:
        return self._session_token

    @property
    def token_type(self) -> TokenType:
        return self._token_type


class RESTRoot(Root):
    """Root that can work either with Session/SnowflakeConnection like other resources or with RESTConnection.

    Can only be used for SparkConnectResource instantiation. All other resources must use the normal snowflake.core.Root
    """

    def __init__(
        self,
        connection: Union[RESTConnection, SnowflakeConnection, "Session"],
        root_config: Optional[RootConfiguration] = None,
    ) -> None:
        self._restConnection: Optional[RESTConnection] = None
        if isinstance(connection, RESTConnection):
            # DO NOT call super.__init__() as it expects either SnowflakeConnection or Session
            self._restConnection = connection
            logger.info("New root object was created for %r", connection)
            self._refresh_parameters()
            # In stored procedures connection.rest doesn't have _host. Let's set the base-case here.
            self._hostname = connection.host
            if self._hostname is not None and self._parameters.fix_hostname:
                # Fix hostname, if desired. This is done by replacing _ (underscores) with - (dashes)
                #  in the first part of the hostname.
                self._hostname = _fix_hostname(self._hostname)
            self._root_config = root_config or RootConfiguration()
        else:
            super().__init__(connection, root_config)

    @property
    def session(self) -> "Session":
        if self._restConnection is not None:
            raise SnowflakePythonError("REST connection does not have Session instance associated with it")
        return super().session

    @property
    def _session_token(self) -> Optional[str]:
        if self._restConnection is not None:
            return self._restConnection.session_token
        return super()._session_token

    @property
    def _master_token(self) -> Optional[str]:
        if self._restConnection is not None:
            return None
        return super()._master_token

    @property
    def token_type(self) -> TokenType:
        if self._restConnection is not None:
            return self._restConnection.token_type
        return super().token_type

    @property
    def external_session_id(self) -> Optional[str]:
        if self._restConnection is not None:
            return self._restConnection.external_session_id
        return super().external_session_id



from __future__ import absolute_import

# flake8: noqa

# import apis into api package
from snowflake.core.catalog_integration._generated.api.catalog_integration_api import CatalogIntegrationApi

__all__ = [
    "CatalogIntegrationApi",
]



import pytest

from snowflake.core.database_role import ContainingScope, Securable
from snowflake.core.table import Table, TableColumn
from tests.integ.database_role.utils import assert_basic_grant, clear_default_grant
from tests.utils import random_string


pytestmark = pytest.mark.min_sf_ver("8.39.0")


@pytest.mark.use_accountadmin
def test_iter_grant_to(
    database_roles_in_test_database,
    test_database_role_name,
    test_database_role_name_2,
    test_database_for_grant_name,
    test_schema_for_grant_name,
    test_table_for_grant_name,
    tables_in_test_database,
):
    # test grant database role becasue in this we will get the securable database name
    database_roles_in_test_database[test_database_role_name].grant_role(
        role_type="DATABASE ROLE", role=Securable(name=test_database_role_name_2, database=test_database_for_grant_name)
    )
    grants_list = list(database_roles_in_test_database[test_database_role_name].iter_grants_to())
    assert len(grants_list) == 2
    grants_list = clear_default_grant(grants_list)
    grant = grants_list[0]
    assert_basic_grant(grant)
    assert not grant.grant_option
    assert grant.securable.name == test_database_role_name_2.upper()
    assert grant.securable.database == test_database_for_grant_name.upper()
    assert grant.securable_type == "DATABASE ROLE"

    # testing limit
    grants_list = list(database_roles_in_test_database[test_database_role_name].iter_grants_to(show_limit=1))
    assert len(grants_list) == 1

    database_roles_in_test_database[test_database_role_name].revoke_role(
        role_type="DATABASE ROLE", role=Securable(name=test_database_role_name_2, database=test_database_for_grant_name)
    )

    # grant on a schema here we will get the securable name as well as schema name
    database_roles_in_test_database[test_database_role_name].grant_privileges(
        privileges=["CREATE TASK"],
        securable_type="SCHEMA",
        securable=Securable(database=test_database_for_grant_name, name=test_schema_for_grant_name),
    )
    grants_list = list(database_roles_in_test_database[test_database_role_name].iter_grants_to())
    assert len(grants_list) == 2
    grants_list = clear_default_grant(grants_list)
    grant = grants_list[0]
    assert_basic_grant(grant)
    assert grant.securable.name == test_schema_for_grant_name.upper()
    assert grant.securable_type == "SCHEMA"
    assert grant.securable.database == test_database_for_grant_name

    database_roles_in_test_database[test_database_role_name].revoke_privileges(
        privileges=["CREATE TASK"],
        securable_type="SCHEMA",
        securable=Securable(database=test_database_for_grant_name, name=test_schema_for_grant_name),
    )

    # grant on a table
    database_roles_in_test_database[test_database_role_name].grant_privileges(
        privileges=["SELECT"],
        securable_type="TABLE",
        securable=Securable(
            database=test_database_for_grant_name, schema=test_schema_for_grant_name, name=test_table_for_grant_name
        ),
    )
    grants_list = list(database_roles_in_test_database[test_database_role_name].iter_grants_to())
    assert len(grants_list) == 2
    grants_list = clear_default_grant(grants_list)
    grant = grants_list[0]
    assert_basic_grant(grant)
    assert grant.securable.name == test_table_for_grant_name.upper()
    assert grant.securable_type == "TABLE"
    assert grant.securable.database == test_database_for_grant_name.upper()
    assert grant.securable.var_schema == test_schema_for_grant_name.upper()

    database_roles_in_test_database[test_database_role_name].revoke_privileges(
        privileges=["SELECT"],
        securable_type="TABLE",
        securable=Securable(
            database=test_database_for_grant_name, schema=test_schema_for_grant_name, name=test_table_for_grant_name
        ),
    )

    # all future tables in a schema
    database_roles_in_test_database[test_database_role_name].grant_future_privileges(
        privileges=["INSERT"],
        securable_type="TABLE",
        containing_scope=ContainingScope(database=test_database_for_grant_name, schema=test_schema_for_grant_name),
    )

    grants_list = list(database_roles_in_test_database[test_database_role_name].iter_grants_to())
    assert len(grants_list) == 1

    try:
        table_name = random_string(5, "test_grant_future_privileges_table_")
        columns = [TableColumn(name="col1", datatype="int"), TableColumn(name="col2", datatype="string")]
        test_table = Table(name=table_name, columns=columns)
        tables_in_test_database.create(test_table)

        grants_list = list(database_roles_in_test_database[test_database_role_name].iter_grants_to())
        assert len(grants_list) == 2
        grants_list = clear_default_grant(grants_list)
        grant = grants_list[0]
        assert_basic_grant(grant)
        assert grant.securable.name == table_name.upper()
        assert grant.securable_type == "TABLE"
        assert grant.securable.database == test_database_for_grant_name.upper()
        assert grant.securable.var_schema == test_schema_for_grant_name.upper()
    finally:
        tables_in_test_database[table_name].drop()


@pytest.mark.use_accountadmin
def test_iter_future_grant_to(
    database_roles_in_test_database, test_database_role_name, test_database_for_grant_name, test_schema_for_grant_name
):
    assert len(list(database_roles_in_test_database[test_database_role_name].iter_future_grants_to())) == 0
    # all future tables in a schema
    database_roles_in_test_database[test_database_role_name].grant_future_privileges(
        privileges=["INSERT"],
        securable_type="TABLE",
        containing_scope=ContainingScope(database=test_database_for_grant_name, schema=test_schema_for_grant_name),
    )

    grants_list = list(database_roles_in_test_database[test_database_role_name].iter_future_grants_to())
    assert len(grants_list) == 1
    grant = grants_list[0]
    assert_basic_grant(grant, False)  # future grants do not have granted_by
    assert grant.securable.name == '"<TABLE>"'
    assert grant.securable_type == "TABLE"
    assert grant.securable.database == test_database_for_grant_name.upper()
    assert grant.securable.var_schema == test_schema_for_grant_name.upper()

    database_roles_in_test_database[test_database_role_name].grant_future_privileges(
        privileges=["UPDATE"],
        securable_type="TABLE",
        containing_scope=ContainingScope(database=test_database_for_grant_name, schema=test_schema_for_grant_name),
    )

    grants_list = list(database_roles_in_test_database[test_database_role_name].iter_future_grants_to())
    assert len(grants_list) == 2

    # testing limit
    grants_list = list(database_roles_in_test_database[test_database_role_name].iter_future_grants_to(show_limit=1))
    assert len(grants_list) == 1



from unittest import mock

import pytest

from snowflake.core import PollingOperation
from snowflake.core.cortex.lite_agent_service import AgentRunRequest, CortexAgentService

from ...utils import BASE_URL, extra_params


API_CLIENT_REQUEST = "snowflake.core._generated.api_client.ApiClient.request"


@pytest.fixture
def agent_service(fake_root):
    return CortexAgentService(fake_root)


def test_run(fake_root, agent_service):
    args = (fake_root, "POST", BASE_URL + "/cortex/agent:run")
    kwargs = extra_params(
        body={
            "model": "my_model",
            "messages": [{"role": "", "content": []}],
            "tool_choice": None,
            "origin_application": "external",
        },
        _preload_content=False,
    )

    with mock.patch(API_CLIENT_REQUEST) as mocked_request:
        op = agent_service.run_async(AgentRunRequest(model="my_model", messages=[{"role": "", "content": []}]))
        assert isinstance(op, PollingOperation)
        op.result()
    mocked_request.assert_called_once_with(*args, **kwargs)



"""
Fixtures for preview feature filtering tests.

These tests validate that the code generator correctly filters out
preview endpoints and models based on x-sf-enabling-parameter.
"""

import sys
from pathlib import Path

import pytest
from unittest.mock import Mock

# Add codegen folder to path so 'tests.preview_test._generated' imports work
_codegen_dir = Path(__file__).parent.parent
if str(_codegen_dir) not in sys.path:
    sys.path.insert(0, str(_codegen_dir))

_GENERATED_CODE_MISSING_MSG = """
Generated test code not found.

Error: {error}
"""


@pytest.fixture(scope="module")
def mock_root():
    return Mock()


@pytest.fixture(scope="module")
def mock_resource_class():
    return Mock()


@pytest.fixture(scope="module")
def mock_sproc_client():
    return Mock()


@pytest.fixture(scope="module")
def default_api(mock_root, mock_resource_class, mock_sproc_client):
    try:
        from tests.preview_test._generated.api.default_api import DefaultApi

        return DefaultApi(root=mock_root, resource_class=mock_resource_class, sproc_client=mock_sproc_client)
    except ImportError as e:
        pytest.fail(_GENERATED_CODE_MISSING_MSG.format(error=e))


@pytest.fixture(scope="module")
def default_api_class():
    try:
        from tests.preview_test._generated.api.default_api import DefaultApi

        return DefaultApi
    except ImportError as e:
        pytest.fail(_GENERATED_CODE_MISSING_MSG.format(error=e))


@pytest.fixture(scope="module")
def models_module():
    try:
        from tests.preview_test._generated import models

        return models
    except ImportError as e:
        pytest.fail(_GENERATED_CODE_MISSING_MSG.format(error=e))



# coding: utf-8

# flake8: noqa
"""
Snowflake Stream API.

The Snowflake Stream API is a REST API that you can use to access, update, and perform certain actions on Stream resource in a Snowflake database.  # noqa: E501

The version of the OpenAPI document: 0.0.1
Contact: support@snowflake.com
Generated by: https://openapi-generator.tech

Do not edit this file manually.
"""

from __future__ import absolute_import

# import models into model package
from snowflake.core.stream._generated.models.error_response import ErrorResponse
from snowflake.core.stream._generated.models.point_of_time import PointOfTime
from snowflake.core.stream._generated.models.point_of_time_offset import PointOfTimeOffset
from snowflake.core.stream._generated.models.point_of_time_statement import PointOfTimeStatement
from snowflake.core.stream._generated.models.point_of_time_stream import PointOfTimeStream
from snowflake.core.stream._generated.models.point_of_time_timestamp import PointOfTimeTimestamp
from snowflake.core.stream._generated.models.stream import Stream
from snowflake.core.stream._generated.models.stream_clone import StreamClone
from snowflake.core.stream._generated.models.stream_source import StreamSource
from snowflake.core.stream._generated.models.stream_source_external_table import StreamSourceExternalTable
from snowflake.core.stream._generated.models.stream_source_stage import StreamSourceStage
from snowflake.core.stream._generated.models.stream_source_table import StreamSourceTable
from snowflake.core.stream._generated.models.stream_source_view import StreamSourceView
from snowflake.core.stream._generated.models.success_accepted_response import SuccessAcceptedResponse
from snowflake.core.stream._generated.models.success_response import SuccessResponse
from snowflake.core.stream._generated.models.tag_assignment import TagAssignment
from snowflake.core.stream._generated.models.tag_reference import TagReference

__all__ = [
    "ErrorResponse",
    "PointOfTime",
    "PointOfTimeOffset",
    "PointOfTimeStatement",
    "PointOfTimeStream",
    "PointOfTimeTimestamp",
    "Stream",
    "StreamClone",
    "StreamSource",
    "StreamSourceExternalTable",
    "StreamSourceStage",
    "StreamSourceTable",
    "StreamSourceView",
    "SuccessAcceptedResponse",
    "SuccessResponse",
    "TagAssignment",
    "TagReference",
]



# coding: utf-8

# flake8: noqa

"""
Cortex Embed API.

OpenAPI 3.0 specification for the Cortex Embed REST API  # noqa: E501

The version of the OpenAPI document: 0.1.0
Contact: support@snowflake.com
Generated by: https://openapi-generator.tech

Do not edit this file manually.
"""

from __future__ import absolute_import

__version__ = "1.0.0"

# import apis into sdk package
from snowflake.core.cortex.embed_service._generated.api.cortex_embed_api import CortexEmbedApi

# import models into sdk package
from snowflake.core.cortex.embed_service._generated.models.embed_request import EmbedRequest
from snowflake.core.cortex.embed_service._generated.models.embed_response import EmbedResponse
from snowflake.core.cortex.embed_service._generated.models.embed_response_data_inner import EmbedResponseDataInner
from snowflake.core.cortex.embed_service._generated.models.embed_response_usage import EmbedResponseUsage
from snowflake.core.cortex.embed_service._generated.models.error_response import ErrorResponse



"""Manages Snowflake Privileges.

Example:
    >>> root.grants.grant(Grant(
    >>>        grantee=Grantees.role(name=role_name),
    >>>        securable=Securables.current_account,
    >>>        privileges=[Privileges.create_database]))
"""

from ._grant import Grant
from ._grantee import Grantee, Grantees
from ._grants import Grants
from ._privileges import Privileges
from ._securables import Securable, Securables


__all__ = ["Grant", "Grants", "Grantee", "Grantees", "Privileges", "Securable", "Securables"]



# coding: utf-8

# flake8: noqa
"""
Snowflake User API.

The Snowflake User API is a REST API that you can use to access, update, and perform certain action on Users in a Snowflake database.  # noqa: E501

The version of the OpenAPI document: 0.0.1
Contact: support@snowflake.com
Generated by: https://openapi-generator.tech

Do not edit this file manually.
"""

from __future__ import absolute_import

# import models into model package
from snowflake.core.user._generated.models.containing_scope import ContainingScope
from snowflake.core.user._generated.models.error_response import ErrorResponse
from snowflake.core.user._generated.models.grant import Grant
from snowflake.core.user._generated.models.securable import Securable
from snowflake.core.user._generated.models.success_accepted_response import SuccessAcceptedResponse
from snowflake.core.user._generated.models.success_response import SuccessResponse
from snowflake.core.user._generated.models.tag_assignment import TagAssignment
from snowflake.core.user._generated.models.tag_reference import TagReference
from snowflake.core.user._generated.models.user import User

__all__ = [
    "ContainingScope",
    "ErrorResponse",
    "Grant",
    "Securable",
    "SuccessAcceptedResponse",
    "SuccessResponse",
    "TagAssignment",
    "TagReference",
    "User",
]



import copy

import pytest

from snowflake.core import CreateMode
from snowflake.core.exceptions import ConflictError
from tests.integ.utils import random_string

from .conftest import (
    test_sequence_minimal_template,
    test_sequence_template,
)


def test_create_and_fetch(sequences):
    name = random_string(10, "test_sequence_create_and_fetch_")
    sequence_handle = sequences[name]

    try:
        sequence = copy.deepcopy(test_sequence_template)
        sequence.name = name
        sequences.create(sequence)

        fetched_sequence = sequence_handle.fetch()

        assert fetched_sequence.name == name.upper()
        assert fetched_sequence.start == 2
        assert fetched_sequence.increment == 2
        assert fetched_sequence.ordered is True
        assert fetched_sequence.comment == "Test sequence"
        assert fetched_sequence.database_name == sequences.database.name.upper()
        assert fetched_sequence.schema_name == sequences.schema.name.upper()
    finally:
        sequence_handle.drop(if_exists=True)


def test_create_and_fetch_minimal(sequences):
    name = random_string(10, "test_sequence_create_and_fetch_")
    sequence_handle = sequences[name]

    try:
        sequence = copy.deepcopy(test_sequence_minimal_template)
        sequence.name = name
        sequences.create(sequence)

        fetched_sequence = sequence_handle.fetch()

        assert fetched_sequence.name == name.upper()
        assert fetched_sequence.start == 1
        assert fetched_sequence.increment == 1
        assert fetched_sequence.ordered is False
        assert fetched_sequence.comment is None
        assert fetched_sequence.database_name == sequences.database.name.upper()
        assert fetched_sequence.schema_name == sequences.schema.name.upper()
    finally:
        sequence_handle.drop(if_exists=True)


def test_create_and_fetch_create_modes(sequences):
    name = random_string(10, "test_sequence_create_and_fetch_")
    sequence_handle = sequences[name]

    try:
        sequence = copy.deepcopy(test_sequence_template)
        sequence.name = name
        sequence.comment = "First version"
        sequences.create(sequence, mode=CreateMode.error_if_exists)
        assert sequence_handle.fetch().comment == "First version"

        with pytest.raises(ConflictError):
            sequences.create(sequence, mode=CreateMode.error_if_exists)

        sequence.comment = "Second version"
        sequences.create(sequence, mode=CreateMode.or_replace)
        assert sequence_handle.fetch().comment == "Second version"

        sequence.comment = "Should not change"
        sequences.create(sequence, mode=CreateMode.if_not_exists)
        assert sequence_handle.fetch().comment == "Second version"
    finally:
        sequence_handle.drop(if_exists=True)



import platform

from types import SimpleNamespace
from unittest import mock
from unittest.mock import MagicMock, patch

import pytest

from snowflake.connector.telemetry import TelemetryData
from snowflake.connector.telemetry import TelemetryField as ConnectorTelemetryField
from snowflake.core._common import ObjectReferenceMixin
from snowflake.core._internal.telemetry import ApiTelemetryClient, _TelemetryEventBuilder, api_telemetry
from snowflake.core._internal.utils import TelemetryField
from snowflake.core.exceptions import APIError
from snowflake.core.version import __version__ as VERSION


class TestApiTelemetryClient:
    """Test suite for ApiTelemetryClient class."""

    @pytest.fixture
    def mock_connection(self):
        """Create a mock SnowflakeConnection for testing."""
        conn = MagicMock()
        conn._telemetry = MagicMock()
        return conn

    @pytest.fixture
    def telemetry_client(self, mock_connection):
        """Create an ApiTelemetryClient instance with mocked dependencies."""
        return ApiTelemetryClient(mock_connection)

    @pytest.fixture
    def mock_connection_no_telemetry(self):
        """Create a mock SnowflakeConnection without telemetry for testing."""
        conn = MagicMock()
        conn._telemetry = None
        return conn

    def test_init_with_telemetry_enabled(self, mock_connection):
        """Test initialization when telemetry is enabled."""
        with patch("snowflake.core._internal.telemetry.is_running_inside_stored_procedure", return_value=False):
            client = ApiTelemetryClient(mock_connection)
            assert client.telemetry == mock_connection._telemetry

    def test_init_with_telemetry_disabled_in_stored_procedure(self, mock_connection):
        """Test initialization when running inside a stored procedure (telemetry disabled)."""
        with patch("snowflake.core._internal.telemetry.is_running_inside_stored_procedure", return_value=True):
            client = ApiTelemetryClient(mock_connection)
            assert client.telemetry is None

    def test_init_with_no_telemetry_client(self, mock_connection_no_telemetry):
        """Test initialization when connection has no telemetry client."""
        with patch("snowflake.core._internal.telemetry.is_running_inside_stored_procedure", return_value=False):
            client = ApiTelemetryClient(mock_connection_no_telemetry)

            assert client.telemetry is None

    def test_send_with_telemetry_enabled(self, telemetry_client):
        """Test _send method when telemetry is enabled."""
        test_message = {"test": "data"}
        timestamp = 12345

        telemetry_client._send(test_message, timestamp)

        telemetry_client.telemetry.try_add_log_to_batch.assert_called_once()
        call_args = telemetry_client.telemetry.try_add_log_to_batch.call_args[0][0]
        assert isinstance(call_args, TelemetryData)
        assert call_args.message == test_message
        assert call_args.timestamp == timestamp

    def test_send_with_no_telemetry(self, mock_connection_no_telemetry):
        """Test send method when telemetry is disabled."""
        with patch("snowflake.core._internal.telemetry.is_running_inside_stored_procedure", return_value=False):
            client = ApiTelemetryClient(mock_connection_no_telemetry)

            # Should not raise an exception and should do nothing
            client.safe_send({"test": "data"})
            # No assertions needed as method should return early

    def test_event_builder_usage_build(self, telemetry_client):
        """Test building a usage telemetry event."""
        builder = _TelemetryEventBuilder(class_name="TestClass", func_name="test_function")
        # Current implementation expects _data to be a mapping when building
        builder._data = {}
        result = builder.usage_event()

        assert result[ConnectorTelemetryField.KEY_SOURCE.value] == "snowflake.core"
        assert result[TelemetryField.KEY_VERSION.value] == VERSION
        assert result[TelemetryField.KEY_PYTHON_VERSION.value] == platform.python_version()
        assert result[TelemetryField.KEY_OS.value] == platform.system()
        assert result[ConnectorTelemetryField.KEY_TYPE.value] == "python_api"
        assert result[TelemetryField.KEY_DATA.value]["class_name"] == "TestClass"
        assert result[TelemetryField.KEY_DATA.value][TelemetryField.KEY_FUNC_NAME.value] == "test_function"

    def test_event_builder_exception_generic(self):
        """Test exception event building for generic exception."""
        builder = _TelemetryEventBuilder(class_name="TestClass", func_name="test_function")
        result = builder.exception_event(ValueError("Test error"))

        assert result[ConnectorTelemetryField.KEY_TYPE.value] == "python_api_exception"
        data_section = result[TelemetryField.KEY_DATA.value]
        assert data_section["exception_type"] == "ValueError"

    def test_event_builder_exception_api_error(self):
        """Test exception event building for APIError subclass."""

        class MockAPIError(APIError):
            def __init__(self):
                self.status = 400
                self.reason = "Bad Request"

            def get_request_info(self):
                # Include 'message' to satisfy APIError.__str__ path
                return {"message": "", "request_id": "test_request_id", "error_code": "test_error_code"}

        api_error = MockAPIError()
        builder = _TelemetryEventBuilder(class_name="TestClass", func_name="test_function")
        result = builder.exception_event(api_error)

        assert result[ConnectorTelemetryField.KEY_SOURCE.value] == "snowflake.core"
        assert result[TelemetryField.KEY_VERSION.value] == VERSION
        assert result[TelemetryField.KEY_PYTHON_VERSION.value] == platform.python_version()
        assert result[TelemetryField.KEY_OS.value] == platform.system()
        assert result[ConnectorTelemetryField.KEY_TYPE.value] == "python_api_exception"
        assert result[TelemetryField.KEY_DATA.value]["class_name"] == "TestClass"
        assert result[TelemetryField.KEY_DATA.value][TelemetryField.KEY_FUNC_NAME.value] == "test_function"
        assert TelemetryField.KEY_CI_ENVIRONMENT_TYPE.value in result

        data = result[TelemetryField.KEY_DATA.value]
        assert data["exception_type"] == "MockAPIError"
        assert data["http_code"] == 400
        assert data["request_id"] == "test_request_id"
        assert data["error_code"] == "test_error_code"
        # pragma: allowlist nextline secret
        assert data["exception_sha256"] == "65da287bf807d02bbd7dcef6bf879d31f2fb167dfb8bbc4ceff45176dc52d61a"
        assert data["is_python_api_error"] is True

    def test_event_builder_exception_none(self):
        """Test exception event building when exception is None."""
        builder = _TelemetryEventBuilder(class_name="TestClass", func_name="test_function")
        result = builder.exception_event(None)  # type: ignore[arg-type]
        data = result[TelemetryField.KEY_DATA.value]
        assert data["exception_type"] == "NoneType"
        assert data["is_python_api_error"] is False


class _DummyCollection:
    def __init__(self, root):
        # minimal _api with api_client attribute to satisfy decorator
        self._api = SimpleNamespace(api_client=object())
        self._root = root

    @property
    def root(self):
        return self._root


class _DummyResource(ObjectReferenceMixin[object]):
    def __init__(self, telemetry_client):
        root = SimpleNamespace(_telemetry_client=telemetry_client)
        self.collection = _DummyCollection(root)

    @api_telemetry
    def success_method(self):
        return "ok"

    @api_telemetry
    def failing_method(self):
        raise ValueError("boom")


def test_api_telemetry_sends_usage_event():
    telemetry_client = SimpleNamespace(safe_send=mock.Mock())
    res = _DummyResource(telemetry_client)

    assert res.success_method() == "ok"

    telemetry_client.safe_send.assert_called_once()
    event = telemetry_client.safe_send.call_args[0][0]
    assert event["type"] == "python_api"
    assert event["source"] == "snowflake.core"
    assert event["data"]["class_name"] == "_DummyResource"
    assert event["data"]["func_name"] == "success_method"


def test_api_telemetry_sends_exception_event_on_error():
    telemetry_client = SimpleNamespace(safe_send=mock.Mock())
    res = _DummyResource(telemetry_client)

    with pytest.raises(ValueError):
        res.failing_method()

    # First usage event, then exception event
    assert telemetry_client.safe_send.call_count == 2
    _, exc_call = telemetry_client.safe_send.call_args_list
    exc_event = exc_call[0][0]
    assert exc_event["type"] == "python_api_exception"
    assert exc_event["data"]["class_name"] == "_DummyResource"
    assert exc_event["data"]["func_name"] == "failing_method"
    assert exc_event["data"]["exception_type"] == "ValueError"



"""
Snowflake Notification Integration API.

The Snowflake Notification Integration API is a REST API that you can use to access, update, and perform certain actions on Notification Integration resource in a Snowflake database.  # noqa: E501

The version of the OpenAPI document: 0.0.1
Contact: support@snowflake.com
Generated by: https://openapi-generator.tech

Do not edit this file manually.
"""

from __future__ import annotations

import json
import pprint
import re  # noqa: F401

from typing import Any, Optional

from pydantic import ConfigDict, StrictStr

from snowflake.core.notification_integration._generated.models.notification_hook import (
    NotificationHook,
)


class NotificationQueueAwsSnsOutbound(NotificationHook):
    """A model object representing the NotificationQueueAwsSnsOutbound resource.

    Constructs an object of type NotificationQueueAwsSnsOutbound with the provided properties.

    Parameters
    __________
    aws_sns_topic_arn : str, optional
        Amazon Resource Name (ARN) of the Amazon SNS (SNS) topic to which notifications are pushed.
    aws_sns_role_arn : str, optional
        ARN of the IAM role that has permissions to publish messages to the SNS topic.
    sf_aws_iam_user_arn : str, optional
        ARN for the Snowflake IAM user created for your account — **Read-only:** *any user-provided value will be ignored.*
    sf_aws_external_id : str, optional
        External ID for the Snowflake IAM user created for your account — **Read-only:** *any user-provided value will be ignored.*
    """

    aws_sns_topic_arn: Optional[StrictStr] = None

    aws_sns_role_arn: Optional[StrictStr] = None

    sf_aws_iam_user_arn: Optional[StrictStr] = None

    sf_aws_external_id: Optional[StrictStr] = None

    __properties = ["type"]

    model_config = ConfigDict(
        validate_by_name=True,
        validate_assignment=True,
    )

    def to_str(self) -> str:
        """Returns the string representation of the model using alias."""
        return pprint.pformat(self.dict(by_alias=True))

    def to_json(self) -> str:
        """Returns the JSON representation of the model using alias."""
        return json.dumps(self.to_dict())

    @classmethod
    def from_json(cls, json_str: str) -> NotificationQueueAwsSnsOutbound:
        """Create an instance of NotificationQueueAwsSnsOutbound from a JSON string."""
        return cls.from_dict(json.loads(json_str))

    def to_dict(
        self,
        hide_readonly_properties: bool = False,
    ) -> dict[str, Any]:
        """Returns the dictionary representation of the model using alias."""
        exclude_properties = set()

        if hide_readonly_properties:
            exclude_properties.update(
                {
                    "sf_aws_iam_user_arn",
                    "sf_aws_external_id",
                }
            )

        _dict = self.model_dump(serialize_as_any=True, by_alias=True, exclude=exclude_properties, exclude_none=True)

        _dict["type"] = NotificationHook.get_child_model_discriminator_value("NotificationQueueAwsSnsOutbound")

        return _dict

    def to_dict_without_readonly_properties(self) -> dict[str, Any]:
        """Return the dictionary representation of the model without readonly properties."""
        return self.to_dict(hide_readonly_properties=True)

    @classmethod
    def from_dict(cls, obj: dict) -> NotificationQueueAwsSnsOutbound:
        """Create an instance of NotificationQueueAwsSnsOutbound from a dict."""
        if obj is None:
            return None

        if type(obj) is not dict:
            return NotificationQueueAwsSnsOutbound.model_validate(obj)

        _obj = NotificationQueueAwsSnsOutbound.model_validate(
            {
                "aws_sns_topic_arn": obj.get("aws_sns_topic_arn"),
                "aws_sns_role_arn": obj.get("aws_sns_role_arn"),
                "sf_aws_iam_user_arn": obj.get("sf_aws_iam_user_arn"),
                "sf_aws_external_id": obj.get("sf_aws_external_id"),
            }
        )

        return _obj


class NotificationQueueAwsSnsOutboundModel(NotificationHook):
    def __init__(
        self,  # optional properties
        aws_sns_topic_arn: Optional[str] = None,
        aws_sns_role_arn: Optional[str] = None,
        sf_aws_iam_user_arn: Optional[str] = None,
        sf_aws_external_id: Optional[str] = None,
    ):
        """A model object representing the NotificationQueueAwsSnsOutbound resource.

        Constructs an object of type NotificationQueueAwsSnsOutbound with the provided properties.

        Parameters
        __________
        aws_sns_topic_arn : str, optional
            Amazon Resource Name (ARN) of the Amazon SNS (SNS) topic to which notifications are pushed.
        aws_sns_role_arn : str, optional
            ARN of the IAM role that has permissions to publish messages to the SNS topic.
        sf_aws_iam_user_arn : str, optional
            ARN for the Snowflake IAM user created for your account.
        sf_aws_external_id : str, optional
            External ID for the Snowflake IAM user created for your account.
        """
        super().__init__()
        self.aws_sns_topic_arn = aws_sns_topic_arn
        self.aws_sns_role_arn = aws_sns_role_arn
        self.sf_aws_iam_user_arn = sf_aws_iam_user_arn
        self.sf_aws_external_id = sf_aws_external_id

    __properties = ["type"]

    def __repr__(self) -> str:
        return repr(self._to_model())

    def _to_model(self):
        return NotificationQueueAwsSnsOutbound(
            aws_sns_topic_arn=self.aws_sns_topic_arn,
            aws_sns_role_arn=self.aws_sns_role_arn,
            sf_aws_iam_user_arn=self.sf_aws_iam_user_arn,
            sf_aws_external_id=self.sf_aws_external_id,
        )

    @classmethod
    def _from_model(cls, model) -> NotificationQueueAwsSnsOutboundModel:
        return NotificationQueueAwsSnsOutboundModel(
            aws_sns_topic_arn=model.aws_sns_topic_arn,
            aws_sns_role_arn=model.aws_sns_role_arn,
            sf_aws_iam_user_arn=model.sf_aws_iam_user_arn,
            sf_aws_external_id=model.sf_aws_external_id,
        )

    def to_dict(self):
        """Create a dictionary of the properties from a NotificationQueueAwsSnsOutbound.

        This method constructs a dictionary with the key-value entries corresponding to the properties of the NotificationQueueAwsSnsOutbound object.

        Returns
        _______
        dict
            A dictionary object created using the input model.
        """
        return self._to_model().to_dict()

    @classmethod
    def from_dict(cls, obj: dict) -> NotificationQueueAwsSnsOutboundModel:
        """Create an instance of NotificationQueueAwsSnsOutbound from a dict.

        This method constructs a NotificationQueueAwsSnsOutbound object from a dictionary with the key-value pairs of its properties.

        Parameters
        ----------
        obj : dict
            A dictionary whose keys and values correspond to the properties of the resource object.

        Returns
        _______
        NotificationQueueAwsSnsOutbound
            A NotificationQueueAwsSnsOutbound object created using the input dictionary; this will fail if the required properties are missing.
        """
        return cls._from_model(NotificationQueueAwsSnsOutbound.from_dict(obj))


NotificationQueueAwsSnsOutbound._model_class = NotificationQueueAwsSnsOutboundModel



from contextlib import suppress

import pytest

from tests.integ.utils import backup_database_and_schema as _backup_database_and_schema
from tests.integ.utils import backup_warehouse


@pytest.fixture
def backup_database_schema(connection):
    """Reset the current database and schema after a test is complete.

    These 2 resources go hand-in-hand, so they should be backed up together.
    This fixture should be used when a database, or schema is created,
    or used in a test.
    """
    with connection.cursor() as cursor, _backup_database_and_schema(cursor):
        with suppress(Exception):
            yield


@pytest.fixture
def backup_warehouse_fixture(connection):
    """Reset the current warehouse after a test is complete.

    This fixture should be used when a warehouse is created, or used in a test.
    """
    with connection.cursor() as cursor, backup_warehouse(cursor) as current_warehouse:
        with suppress(Exception):
            yield current_warehouse



"""
Snowflake Catalog Integration API.

The Snowflake Catalog Integration API is a REST API that you can use to access, update, and perform certain actions on Catalog Integration resource in a Snowflake database.  # noqa: E501

The version of the OpenAPI document: 0.0.1
Contact: support@snowflake.com
Generated by: https://openapi-generator.tech

Do not edit this file manually.
"""

from __future__ import annotations

import json
import pprint
import re

from datetime import datetime
from typing import Any, Optional

from pydantic import BaseModel, ConfigDict, Field, StrictBool, StrictStr, field_validator
from typing_extensions import Annotated

from snowflake.core.catalog_integration._generated.models.catalog import Catalog, CatalogModel


class CatalogIntegration(BaseModel):
    """A model object representing the CatalogIntegration resource.

    Constructs an object of type CatalogIntegration with the provided properties.

    Parameters
    __________
    name : str
        Name of the catalog integration.
    catalog : Catalog

    table_format : str
        Table format of the catalog.
    enabled : bool
        whether this catalog integration is available to use for Iceberg tables.
    comment : str, optional
        Comment.
    type : str, optional
        Type of the integration. Always CATALOG — **Read-only:** *any user-provided value will be ignored.*
    category : str, optional
        Category of the integration. Always CATALOG — **Read-only:** *any user-provided value will be ignored.*
    created_on : datetime, optional
        Date and time when the catalog integration was created — **Read-only:** *any user-provided value will be ignored.*
    """

    name: Annotated[str, Field(strict=True)]

    catalog: Catalog

    table_format: StrictStr

    enabled: StrictBool

    comment: Optional[StrictStr] = None

    type: Optional[StrictStr] = None

    category: Optional[StrictStr] = None

    created_on: Optional[datetime] = None

    __properties = ["name", "catalog", "table_format", "enabled", "comment", "type", "category", "created_on"]

    @field_validator("name")
    def name_validate_regular_expression(cls, v):
        if not re.match(r"""^\"([^\"]|\"\")+\"|[a-zA-Z_][a-zA-Z0-9_$]*$""", v):
            raise ValueError(r"""must validate the regular expression /^"([^"]|"")+"|[a-zA-Z_][a-zA-Z0-9_$]*$/""")
        return v

    @field_validator("table_format")
    def table_format_validate_enum(cls, v):
        if v not in ("ICEBERG"):
            raise ValueError("must validate the enum values ('ICEBERG')")
        return v

    model_config = ConfigDict(
        validate_by_name=True,
        validate_assignment=True,
    )

    def to_str(self) -> str:
        """Returns the string representation of the model using alias."""
        return pprint.pformat(self.dict(by_alias=True))

    def to_json(self) -> str:
        """Returns the JSON representation of the model using alias."""
        return json.dumps(self.to_dict())

    @classmethod
    def from_json(cls, json_str: str) -> CatalogIntegration:
        """Create an instance of CatalogIntegration from a JSON string."""
        return cls.from_dict(json.loads(json_str))

    def to_dict(
        self,
        hide_readonly_properties: bool = False,
    ) -> dict[str, Any]:
        """Returns the dictionary representation of the model using alias."""
        exclude_properties = set()

        if hide_readonly_properties:
            exclude_properties.update(
                {
                    "type",
                    "category",
                    "created_on",
                }
            )

        _dict = self.model_dump(serialize_as_any=True, by_alias=True, exclude=exclude_properties, exclude_none=True)

        # override the default output from pydantic by calling `to_dict()` of catalog
        if self.catalog:
            _dict["catalog"] = self.catalog.to_dict()

        return _dict

    def to_dict_without_readonly_properties(self) -> dict[str, Any]:
        """Return the dictionary representation of the model without readonly properties."""
        return self.to_dict(hide_readonly_properties=True)

    @classmethod
    def from_dict(cls, obj: dict) -> CatalogIntegration:
        """Create an instance of CatalogIntegration from a dict."""
        if obj is None:
            return None

        if type(obj) is not dict:
            return CatalogIntegration.model_validate(obj)

        _obj = CatalogIntegration.model_validate(
            {
                "name": obj.get("name"),
                "catalog": Catalog.from_dict(obj.get("catalog")) if obj.get("catalog") is not None else None,
                "table_format": obj.get("table_format"),
                "enabled": obj.get("enabled"),
                "comment": obj.get("comment"),
                "type": obj.get("type"),
                "category": obj.get("category"),
                "created_on": obj.get("created_on"),
            }
        )

        return _obj


class CatalogIntegrationModel:
    def __init__(
        self,
        name: str,
        catalog: Catalog,
        table_format: str,
        enabled: bool,
        # optional properties
        comment: Optional[str] = None,
        type: Optional[str] = None,
        category: Optional[str] = None,
        created_on: Optional[datetime] = None,
    ):
        """A model object representing the CatalogIntegration resource.

        Constructs an object of type CatalogIntegration with the provided properties.

        Parameters
        __________
        name : str
            Name of the catalog integration.
        catalog : Catalog

        table_format : str
            Table format of the catalog.
        enabled : bool
            whether this catalog integration is available to use for Iceberg tables.
        comment : str, optional
            Comment.
        type : str, optional
            Type of the integration. Always CATALOG.
        category : str, optional
            Category of the integration. Always CATALOG.
        created_on : datetime, optional
            Date and time when the catalog integration was created.
        """
        self.name = name
        self.catalog = catalog
        self.table_format = table_format
        self.enabled = enabled
        self.comment = comment
        self.type = type
        self.category = category
        self.created_on = created_on

    __properties = ["name", "catalog", "table_format", "enabled", "comment", "type", "category", "created_on"]

    def __repr__(self) -> str:
        return repr(self._to_model())

    def _to_model(self):
        return CatalogIntegration(
            name=self.name,
            catalog=self.catalog._to_model() if self.catalog is not None else None,
            table_format=self.table_format,
            enabled=self.enabled,
            comment=self.comment,
            type=self.type,
            category=self.category,
            created_on=self.created_on,
        )

    @classmethod
    def _from_model(cls, model) -> CatalogIntegrationModel:
        return CatalogIntegrationModel(
            name=model.name,
            catalog=CatalogModel._from_model(model.catalog) if model.catalog else None,
            table_format=model.table_format,
            enabled=model.enabled,
            comment=model.comment,
            type=model.type,
            category=model.category,
            created_on=model.created_on,
        )

    def to_dict(self):
        """Create a dictionary of the properties from a CatalogIntegration.

        This method constructs a dictionary with the key-value entries corresponding to the properties of the CatalogIntegration object.

        Returns
        _______
        dict
            A dictionary object created using the input model.
        """
        return self._to_model().to_dict()

    @classmethod
    def from_dict(cls, obj: dict) -> CatalogIntegrationModel:
        """Create an instance of CatalogIntegration from a dict.

        This method constructs a CatalogIntegration object from a dictionary with the key-value pairs of its properties.

        Parameters
        ----------
        obj : dict
            A dictionary whose keys and values correspond to the properties of the resource object.

        Returns
        _______
        CatalogIntegration
            A CatalogIntegration object created using the input dictionary; this will fail if the required properties are missing.
        """
        return cls._from_model(CatalogIntegration.from_dict(obj))


CatalogIntegration._model_class = CatalogIntegrationModel
